/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;
/*
  A 3d point.
  Compos de trois dimensions x, y, z
  et deux autres u, v non utilises
 */

public class Vertex 
{
	double x;
	double y;
	double z;
	double u;
	double v;


	public Vertex() { this(0, 0, 0); 		//{{INIT_CONTROLS
		//}}
}	

	public Vertex(double x, double y, double z) 
	{
		this(x, y, z, (int) x, (int) y);
	}	

	public Vertex(double x, double y, double z, double u, double v) 
	{
		this.x = x;
		this.y = y;
		this.z = z;
		this.u = u;
		this.v = v;
	}	

	public Vertex(Vertex v) 
	{
		this.x = v.x;
		this.y = v.y;
		this.z = v.z;
		this.u = v.u;
		this.v = v.v;
	}	

	public void rotate(Orient o1) 
	{
		double x, y, z;
		double nx, ny, nz;

		nx = (this.x * o1.yaw.cosine) - (this.z * o1.yaw.sine);
		ny = this.y;
		nz = (this.x * o1.yaw.sine) + (this.z * o1.yaw.cosine);
		x = nx;
		y = (o1.pitch.cosine * ny) + (-o1.pitch.sine * nz);
		z = (o1.pitch.sine * ny) + (o1.pitch.cosine * nz);
		this.x = (x * o1.roll.cosine) + (y * o1.roll.sine);
		this.y = (-x * o1.roll.sine) + (y * o1.roll.cosine);
		this.z = z;
	}

	public void translate(Vertex v) 
	{
		x += v.x;
		y += v.y;
		z += v.z;
	}

	public Vertex negative() 
	{
		return new Vertex(-x, -y, -z);
	}

	public double dotproduct(Vertex b) 
	{
		return ((this.x * b.x) + (this.y * b.y) + (this.z * b.z));
	}

	public double length() 
	{
		return Math.sqrt((x * x) + (y * y) + (z * z));
	}
	
	public String toString() 
	{
		return "Vertex [" + x + ", " + y + ", " + z + "]";
	}
	
	//{{DECLARE_CONTROLS
	//}}
}
