/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.util.zip.*;


/**
  
  Class ANIMATOR

  -------------------------------------------------------
  Usage:
   
  Animator.restart(); 
  do
  {
   int type = Animator.getType();
   Object picture = Animator.get(); // cast further to proper format
   
   switch(type)
   {
    case Animator.GIF:  ... display picture as GIF 
    case Animator.IMAGE: ...display picture as IMAGE
    case Animator.RAW: .... display picture as RAW
    default: break;
   } 
   
  } while(Animator.next());

 --------------------------------------------------------
*/

public class Animator
{
 static public final int BRANCH = 0x10000;
 static public final int IMAGE  = 0;  // Public free Java Image format
 static public final int RAW    = 1;  // Public free AA Raw image format
 static public final int GIF    = 2;  // Private Unisys' format
 static public final int JPG    = 3;  // Public free jpeg format
 static public final int PNG    = 4;  // Public free png format
 static public final int WAV    = 8;
 static public final int AU     = 9; 
 static public final int DOC    = 16;
 
 Vector pictures = new Vector();
 Vector names    = new Vector();
 IVector types   = new IVector();  // Image, gif, raw, etc...
 IVector parents = new IVector();  // Index of parent branch of item
 IVector nexts   = new IVector();  // Index of next image or item
 IVector childs  = new IVector();  // Index of branch if one
 int count = 0;    // Max in vector;
 int index = 0;    // Image index
 int delay = 50;   // Speed

 public String dir = "";
 public String name = null;  
 public String message = "ok";

 public void restart()
 {
  index = 0;
 }

 public void setSlow(int s) { delay = s; } 



 /** Increment index */
 public boolean next() 
 {
  return (++index < count);
 }
 
 /** Return type */
 
 public int getType(int index) { return types.elementAt(index);  }   
 public int getType() { return types.elementAt(index); }   

 /** Return the picture at index */
 public Object getPicture(int i) { return pictures.elementAt(i); }

 /** Return the next picture */
 public Object getPicture() { return pictures.elementAt(index); }
 
 /** Return the picture with this name */
 public Object getPicture(String name)
 {
   int i = 0;
   while(i < count)
   {
    if(((String) names.elementAt(i)).equalsIgnoreCase(name)) return getPicture(i);
    i++; 
   } 
   
  return null;  
 }   
 
 /** Misc */
 
 int next(int index)  { return nexts.elementAt(index); }   
 int child(int index) { return childs.elementAt(index); }


 /**
     Speed: the lower value, the faster
 */
 
 public void setSpeed(int millis)
 {
   delay = millis;  
 }   


 int fieldX = 0;
 int fieldY = 0;
 int fieldW = 0;
 int fieldH = 0;
 
 /**
    Define the animation area
 */   
 public void setField(ImageViewer iv)
 {
  Dimension fieldDim = iv.getSize();
  fieldW = fieldDim.width;
  fieldH = fieldDim.height;
  fieldX = 0;
  fieldY = 0;
 }   
 
 public void clearField(Graphics g)
 {
   g.clearRect(fieldX, fieldY, fieldW, fieldH);
 }   

 /**
    Animate the content on a given graphic context 
 */    
 
     
 public void animate(Graphics g, int x, int y)
 {
   restart(); 
   do
   {
    int type = getType();
    Object picture = (Object) getPicture();
    clearField(g);
   
    switch(type)
    {
     case GIF: 
     case IMAGE: g.drawImage((Image) picture, x, y, null);
                 break;
     case RAW:   Raw raw = (Raw) picture;
                 Image image = raw.getImage();
                 g.drawImage((Image) image, x, y, null);
                 break;
     default:    break;
    } 
    
    try { Thread.sleep(delay); } catch(InterruptedException i) {}
   } while(next());
 }
 
 

 /** 
    Reset vectors 
 */
 
 public void clear(int size)
 {
  pictures = new Vector(size);
  names  = new Vector(size);
  types  = new IVector(size);
  parents = new IVector(size);  
  nexts   = new IVector(size);
  childs  = new IVector(size);
  count = 0;
  index = 0;
  name = null;
  dir = "";
 }   

 public void clear()  { clear(16); }   
 

 
 int parent = 0;
 int previous = 0;
 int newfolder = -1;

 /** 
     Cre folder
     Il sera parent du prochain lment ajout
 */
 void addBranch(String name, int parent)
 {
  pictures.addElement(null);
  names.addElement(name);
  types.addElement(BRANCH);
  parents.addElement(parent);
  nexts.addElement(0);
  childs.addElement(0);

  newfolder = count++;
 }

 /**
    Cherche folder de ce nom 
    Le cre au besoin
    
    - input: folder name
             the global variable parent (parent of this folder) 
 */
 int findPrevious(String folder)
 {
  int index = count - 1; 
  while(index >= 0)
  {
   if(types.elementAt(index) == BRANCH) 
    if(folder.equalsIgnoreCase((String) names.elementAt(index)))
     return index;

   index--;     
  } 

  // Set previous having count as sibling
  // starting at second entry
  if(count > 0)
  nexts.setElementAt(count, previous);
  addBranch(folder, newfolder);
  //System.out.println("added " + folder);
  return index;
 }   


 /**
   Trouve parent avec le chemin 
   Input: a folder or a path "dir" as parameter
          the global variable "newFolder" used by findPrevious()
   Ouput: the parent the image, that corresponds
          to the last folder in the path (in "newFolder")
   
   - si le nom est un simple folder, rechercher si existe 
     avec findPrevious() qui le crera sinon
   - si c'est un chemin de folders,
     rechercher si le premier existe
     et appeler rcursivement la fonction avec le reste
 
 */
 
 
 void makeFolder(String dir)
 {
  int i = dir.indexOf(File.separatorChar);
  int previous;
  System.out.println("get parent of " + dir);

  if(i == -1) previous = findPrevious(dir); // is a folder
  else // the name holds a folder name
  {
   // search/create the first  get its number 
   previous = findPrevious(dir.substring(0, i)); 
   //if(newfolder >= 0) System.out.println("makeFolder first -  parent " + names.elementAt(newfolder));

   // recurse with trailing string and parent found  
   makeFolder(dir.substring(i + 1)); 
   return;
  }

  //if(newfolder >= 0) System.out.println("makeFolder passed -  parent " + names.elementAt(newfolder));
 }
 
 
 /** 
    Insert an image into the animation 
 */    
 
 void insert(String entry, Object data)
 {
  // Extract path to evaluate parent node
  int i = entry.lastIndexOf(File.separatorChar);  
  // set the global variable to no new folder
  // if following statements create a new folder
  // this global variable will be updated to its number
  newfolder = -1; 
  if(i > -1)
  {
    //System.out.println("insert - get parent folder for " + entry);
    makeFolder(entry.substring(0, i));  
    entry = entry.substring(i + 1);
  }  

/*
  if(newfolder >= 0)
   System.out.println(entry + " parent " + names.elementAt(newfolder));
  else 
   System.out.println(entry + " parent " + newfolder);
*/  
  
  int type;
  i = entry.lastIndexOf('.');
  String ext = entry.substring(i + 1);
  if(ext.equalsIgnoreCase("Raw")) type = RAW;
  else type = DOC;
  
  pictures.addElement(data);  // Add data (image, etc...)   
  names.addElement(entry);    // Add name
  types.addElement(type);     // Add image type
  parents.addElement(newfolder); // Add parent node number
  nexts.addElement(0);        // Clear next node number
  childs.addElement(0);       // Clear child node number
  
  // Set this node as child or sibling 
  // NB: syntax is setElementAt(value, index)
  // When parent folder created, newfolder is greater then -1
  if(newfolder >= 0)  childs.setElementAt(count, newfolder);
  else nexts.setElementAt(count, previous);

  System.out.println(entry + " no" + count + " parent " + parent +
   " type " + type + 
  " *** next of " + names.elementAt(previous) + 
  " as next " + nexts.elementAt(previous)  );

  previous = count++;    
 }   
 
 
 void insert(String name, byte[] imgdata)
 {
  insert(name, (Object) new Raw(imgdata));
 }   


 /**
     Convert Zip to Java conventions
 */   
 public String zipToJava(String zipentry)
  {
   return zipentry.replace('/' , File.separatorChar);
  }

 /**
    Load an animation from a file
 */


  
 public int load(String archive)
 {
   int counter = 0;
   message = "Ok";
   previous = 0;
   clear();

   try
   {
     FileInputStream fis = new FileInputStream(archive);
     BufferedInputStream bis = new BufferedInputStream(fis);
     ZipInputStream zis = new ZipInputStream(bis);
     ZipEntry ze = null;   
     
     ZipFile zf = new ZipFile(archive);
     //Enumeration e = zf.entries();
     
     while((ze = zis.getNextEntry()) != null)
     {

       String entry = zipToJava(ze.getName());
       //ZipE = e.getNextElement();
       //System.out.println(ze.getName() + " > " + entry);
       // Required to read the size in the central directory,
       // because it is not stored into the local header
       ZipEntry ge = zf.getEntry(entry);
       
       //System.out.println(entry);            
        
       if(ze.isDirectory()) // ignored
       {
       }
       else
       {
        try
        {
         int size = (int) ge.getSize();
         byte[] byteBuffer = new byte[size];
         size = zis.read(byteBuffer);
         //System.out.println("entry " + entry + " comp " + ze.getCompressedSize() + " uncomp " + ze.getSize() + " read " + size);        
         insert(entry, byteBuffer);
         counter++;
        } catch(Exception ae){ ae.printStackTrace(); }  
       } // file, image
     } // while
     
     zis.close();
     bis.close();
     fis.close();
     
     name = archive;
    }
   catch(IOException io)   { message = io.toString(); System.out.println(message);}
   catch(InternalError ie) { message = ie.toString(); System.out.println(message);}          
   
   return counter;
 } // load
 
 
 } // class 
 
 /**
    Associated class to handle array of integers,
    derived from the Java's Vector class
 */
 
 class IVector
 {
    public int elementData[]; // array buffer to values
    public int elementCount;  // number of values
    protected int capacityIncrement = 1; // size of upgrade

    /** Constructs with initial capacity */
    public IVector(int capacity)
    {
	 this.elementData = new int[capacity];
	 this.capacityIncrement = 1;
    		//{{INIT_CONTROLS
		//}}
}

    /** Constructs with default capacity  */
    public IVector() { this(16); }


    /** Increases the capacity if necessary */
    public final synchronized void ensureCapacity(int minCapacity)
    {
	 int oldCapacity = elementData.length;
	 if (minCapacity > oldCapacity)
	  {
	    int oldData[] = elementData;
	    int newCapacity = oldCapacity + capacityIncrement;

        if (newCapacity < minCapacity) { newCapacity = minCapacity; }

	    elementData = new int[newCapacity];
        copy(oldData, elementData, elementCount);	    
	  }
    }


    /** Returns the number of components */
    public final int size() { return elementCount; }


    /** Returns the component at the specified index  */
    public final synchronized int elementAt(int index)
    {
      return elementData[index];
    }


    /** Change the value at the specified index  */
    public final synchronized void setElementAt(int val, int index)
    {
	  elementData[index] = val;
    }

    /** Adds the specified value to the list */
     public final synchronized void addElement(int val)
     {
	  ensureCapacity(elementCount + 1);
	  elementData[elementCount++] = val;
     }

    /*
    public final synchronized void insertElementAt(int val, int index) 
    {
     if(index > elementCount)
     {
      throw new ArrayIndexOutOfBoundsException(index + " > " + elementCount);
      return;
     }

 	 if (index < elementCount) 
	 {
	  ensureCapacity(elementCount + 1);
	  copy(elementData, index, elementData, index + 1, elementCount - index);
	 } 
	 
	  // index = elementCount
	  elementData[index] = val;
	  elementCount++;
	}  
    */


    /** Deletes the item at specified index */
    public final synchronized void removeElementAt(int index)
    {
	 //if (index >= elementCount) return;
	 //if (index < 0) return;

	 int j = elementCount - index - 1;
	 if (j > 0) 
	  copy(elementData, index + 1, elementData, index, j);
	 elementCount--;
	 elementData[elementCount] = 0;
    }

    /** Copy a range to array */
    void copy(int [] src, int sdeb, int [] dst, int ddeb, int len)
    {
     for (int i = 0; i < len; i++)
      dst[i + ddeb] = src[i + sdeb];
    }
    
    /** Copy the whole into array */ 
    void copy(int [] src, int [] dst, int len)
    {
     for (int i = 0; i < len; i++)  dst[i] = src[i];
    }
     
     
    public void clear()  { elementCount = 0;  }    

 	//{{DECLARE_CONTROLS
	//}}
} // IVector


 /**
    Load a tree structur
  */ 
 /* 
 public final boolean load(String path)
 {

  try 
  {
    DataInputStream p = new DataInputStream(new FileInputStream(path));
     
    int size = p.readInt();
    clear(size);
    
    while(size > 0)
    {
     int imageIndex = p.readInt();
     
     int pSize = p.readInt();
     byte rawBytes[] = new byte[pSize];
     p.read(rawBytes);
     
     pictures.addElement((Object) rawBytes);
     names.addElement(p.readLine());     
     types.addElement(p.readInt());
     parents.addElement(p.readInt());    
     nexts.addElement(p.readInt());
     childs.addElement(p.readInt());
    
     size--;
    }
    p.close();
   }
  catch(IOException io) { }
 
  return false;
 }
*/
