/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;

/**
   Convert images to text
*/

// import Disk;
// import VirtualDirect;
// import PalettePanel;

public class DataText
{
 static final int IMAGE = 0;
 static final int JAVA = 1;
 static final int C = 2;
 static final int PASCAL = 3;

 static String eol = System.getProperty("line.separator");

 static DMessageBar status = null;

 static void setStatusBar(DMessageBar dmb)
 {
   status = dmb;
 }

 static void Message(String message)
 {
   if(status != null) status.setText(message);
 }

 static byte signed(int value)
 {
  byte bValue;

  if(value < 128) bValue = (byte) value;
  else bValue = (byte) (256 - value);

  return bValue;
 }

 static void save(Frame parent, VirtualDirect offScreen, String path, String name, int mode)
 {
  FileDialog fd = new FileDialog(parent, "Save source file", FileDialog.SAVE);
  fd.setDirectory(path);
  fd.setFile(name);
  fd.show();

  String dname = fd.getDirectory();
  String fname = fd.getFile();
  if(fname == null) return;

  int orient = Preferences.Inversed ? Raw.BTRUE : Raw.BFALSE;
  int comp   = Preferences.Compression;
  int transp = PalettePanel.transparentIndex;

  switch(mode)
  {
   case IMAGE:
  	   saveJava(offScreen, dname, fname);
  	   break;
   case JAVA:
	   saveJavaRaw(offScreen, dname, fname, orient, comp, transp);
	   break;
   case C:
	   saveC(offScreen, dname, fname, orient, comp, transp);
	   break;
   case PASCAL:
	   savePascal(offScreen, dname, fname, orient, comp, transp);
	   break;
   default: break;

  }


 }

 /** Convertit une image en Java */

 static void saveJava(VirtualDirect offScreen, String dname, String fname)
  {
    int IMAGEWIDTH = offScreen.width;
    int IMAGEHEIGHT = offScreen.height;

    // Ajoute l'extension si absente
    if(fname.lastIndexOf('.') == -1)  fname += ".java";

    //debug("dname "+dname + " fname " +fname );
    Message("Saving image as Image source file " + Path.merge(dname, fname));

       // Enlve l'extension
 	int lio = fname.lastIndexOf('.');
 	String name = fname.substring(0, lio);

 	//debug("dname "+dname + " fname " + fname + " et name " + name);

   	StringBuffer sb = new StringBuffer("// Java Image Code" + eol + eol);

    int m = offScreen.getSize();
        //debug("length = " + PIXELS.length);
    int ret = 0;

        //debug("" + PalettePanel.Transparent);
    int TranspInt = PalettePanel.colorToInt(PalettePanel.transparentColor);

    String nw = name + "Width";
    String nh = name + "Height";

    sb.append("static Image " + name + ";" + eol);
    sb.append("static int " + nw + " = " + (new Integer(IMAGEWIDTH)).toString() + ";" + eol);
    sb.append("static int " + nh + " = " + (new Integer(IMAGEHEIGHT)).toString() + ";" + eol);
   	sb.append("static int[] " + name + "RGB = new int[] {" + eol );

    //debug("Save - size " + IMAGEWIDTH + "x" + IMAGEHEIGHT);

	for (int i = 0; i < m; i++)
	  {
       // Pixel suivent comme entier
	  int pix = offScreen.screen[i];

      // Alpha = opaque si non transparent
      //if(pix != TranspInt) pix |= (255 << 24);

	  sb.append((new Integer(pix)).toString());
      if(i < (m - 1)) sb.append(',');
	  if(++ret >= 5) { sb.append(eol); ret = 0; }
	 }

    sb.append(" };" + eol + eol);
    sb.append(name + " = createImage(new MemoryImageSource(" + nw + ", " + nh + ", " + name + "RGB, 0, " + nw + "));" + eol);

 	Disk.save(dname, fname, sb.toString());
}



 /** Converti Raw en java Raw */

 static void saveJavaRaw(VirtualDirect offScreen, String dname, String fname,
    int direction, int compression, int transparent)
  {
    int IMAGEWIDTH = offScreen.width;
    int IMAGEHEIGHT = offScreen.height;

    // Ajoute l'extension si absente
    if(fname.lastIndexOf('.') == -1)  fname += ".java";

    //debug("dname "+dname + " fname " +fname );
    Message("Saving image as Raw Java source file " + Path.merge(dname, fname));

    // Enlve l'extension
 	int lio = fname.lastIndexOf('.');
 	String name = fname.substring(0, lio);

 	//debug("dname "+dname + " fname " + fname + " et name " + name);

   	StringBuffer sb = new StringBuffer("// Java Code in raw AZ format (10 bytes header)" + eol);
    sb.append("// Pixel format: index in palette as signed byte" + eol + eol);

    int m = offScreen.getSize();
    int ret = 0;


   	sb.append("public static byte[] " + name +"Header" + " = {" + eol  );
   	int wh = IMAGEWIDTH / 256;
   	int wl = IMAGEWIDTH  - wh * 256;
   	int hh = IMAGEHEIGHT / 256;
   	int hl = IMAGEHEIGHT  - hh * 256;

    sb.append(new Integer(wl).toString()  + ",");
    sb.append(new Integer(wh).toString() + ",");
    sb.append(new Integer(hl).toString()  + ",");
    sb.append(new Integer(hh).toString() + ",");

    sb.append(signed(direction) + ",");
    sb.append(signed(compression) + ",");
    sb.append("0,"); // color model
    sb.append("0,"); // palette
    sb.append(signed(transparent) + ",");
    sb.append("0" + eol); // version
    sb.append(" };" + eol + eol);

   	sb.append("public static byte[] " + name + "Code" + " = {" + eol  );

	for (int i = 0; i < m; i++)
     {
	  int pix = offScreen.screen[i];
      int val = PalettePanel.searchIndex(pix);

      if(val < 128) sb.append("" + val);
      else          sb.append("" + (val - 256));

      if(i < (m - 1)) sb.append(',');
	  if(++ret >= 10) { sb.append(eol); ret = 0; }
	 }

    sb.append(" };" + eol + eol);

 	Disk.save(dname, fname, sb.toString());
}



 /** Converti Raw en C */

 static void saveC(VirtualDirect offScreen, String dname, String fname,
    int direction, int compression, int transparent)
  {
    int IMAGEWIDTH = offScreen.width;
    int IMAGEHEIGHT = offScreen.height;

    // Ajoute l'extension si absente
    if(fname.lastIndexOf('.') == -1)  fname += ".cpp";

    //debug("dname "+dname + " fname " +fname );
    Message("Saving image as C source file " + Path.merge(dname, fname));

    // Enlve l'extension
 	int lio = fname.lastIndexOf('.');
 	String name = fname.substring(0, lio);

 	//debug("dname "+dname + " fname " + fname + " et name " + name);

   	StringBuffer sb = new StringBuffer("// C Code in raw AZ format (10 bytes header)" + eol);
    sb.append("// Pixel format: alpha, red, green, blue" + eol + eol);

    int m = offScreen.getSize();
    int ret = 0;

//    int TranspInt = PalettePanel.colorToInt(PalettePanel.transparentColor);

   	sb.append("unsigned char[] " + name + " = {" + eol);
   	int wh = IMAGEWIDTH / 256;
   	int wl = IMAGEWIDTH  - wh * 256;
   	int hh = IMAGEHEIGHT / 256;
   	int hl = IMAGEHEIGHT  - hh * 256;

    sb.append(new Integer(wl).toString()  + ",");
    sb.append(new Integer(wh).toString() + ",");
    sb.append(new Integer(hl).toString()  + ",");
    sb.append(new Integer(hh).toString() + ",");

    sb.append((new Integer(direction)).toString() + ",");
    sb.append((new Integer(compression)).toString() + ",");
    sb.append("0,"); // color model
    sb.append("0,"); // palette
    sb.append((new Integer(transparent)).toString() + ",");
    sb.append("0," + eol); // version


    //debug("Save - size " + IMAGEWIDTH + "x" + IMAGEHEIGHT);

	for (int i = 0; i < m; i++)
     {
	  int pix = offScreen.screen[i];

      // Alpha = opaque si non transparent
      //if(pix != TranspInt) pix |= (255 << 24);
      int a = (pix >> 24) & 255;
      int r = (pix >> 16) & 255;
      int g = (pix >> 8) & 255;
      int b = pix & 255;

	  sb.append((new Integer(a)).toString() + ",");
	  sb.append((new Integer(r)).toString() + ",");
	  sb.append((new Integer(g)).toString() + ",");
	  sb.append((new Integer(b)).toString());
      if(i < (m - 1)) sb.append(',');

	  if(++ret >= 5) { sb.append(eol); ret = 0; }
	}

    sb.append(" };" + eol + eol);

 	Disk.save(dname, fname, sb.toString());
}


 /** Convertit Raw en Pascal */
 static void savePascal(VirtualDirect offScreen, String dname, String fname,
    int direction, int compression, int transparent)
  {
    int IMAGEWIDTH = offScreen.width;
    int IMAGEHEIGHT = offScreen.height;

    // Ajoute l'extension si absente
    if(fname.lastIndexOf('.') == -1)  fname += ".pas";

    //debug("dname "+dname + " fname " +fname );
    Message("Saving image as Pascal source file " + Path.merge(dname, fname));

    // Enlve l'extension
 	int lio = fname.lastIndexOf('.');
 	String name = fname.substring(0, lio);

 	//debug("dname "+dname + " fname " + fname + " et name " + name);

   	StringBuffer sb = new StringBuffer("// Pascal Code in raw AZ format (10 bytes header)" + eol);
    sb.append("// Pixel format: alpha, red, green, blue" + eol + eol);

    int m = offScreen.getSize();
    int ret = 0;

//    int TranspInt = PalettePanel.colorToInt(PalettePanel.transparentColor);

   	int wh = IMAGEWIDTH / 256;
   	int wl = IMAGEWIDTH  - wh * 256;
   	int hh = IMAGEHEIGHT / 256;
   	int hl = IMAGEHEIGHT  - hh * 256;

   	sb.append("Const  " + name + " := Array[0.." + (m * 4 + 10) + "] Of Byte = (" + eol);

    sb.append(new Integer(wl).toString()  + ",");
    sb.append(new Integer(wh).toString() + ",");
    sb.append(new Integer(hl).toString()  + ",");
    sb.append(new Integer(hh).toString() + ",");

    sb.append((new Integer(direction)).toString() + ",");
    sb.append((new Integer(compression)).toString() + ",");
    sb.append("0,"); // color model
    sb.append("0,"); // palette
    sb.append((new Integer(transparent)).toString() + ",");
    sb.append("0," + eol); // version

	for (int i = 0; i < m; i++)
     {
	  int pix = offScreen.screen[i];

      // Alpha = opaque si non transparent
      //if(pix != TranspInt) pix |= (255 << 24);
      int a = (pix >> 24) & 255;
      int r = (pix >> 16) & 255;
      int g = (pix >> 8) & 255;
      int b = pix & 255;

	  sb.append((new Integer(a)).toString() + ",");
	  sb.append((new Integer(r)).toString() + ",");
	  sb.append((new Integer(g)).toString() + ",");
	  sb.append((new Integer(b)).toString());
      if(i < (m - 1)) sb.append(',');

	  if(++ret >= 5) { sb.append(eol); ret = 0; }
	}

    sb.append(" );" + eol + eol);

 	Disk.save(dname, fname, sb.toString());
 }



}
