/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.event.*;


/**
   Painter
   dispose de fonctions de dessin

   Utilise cran virtual associ
   Undo global pour mouseDragged
*/


public class Painter extends Canvas implements MouseListener, ActionListener, MouseMotionListener, KeyListener, FocusListener
{
  static VirtualBig bigScreen;
  static VirtualDirect realScreen;
  static VirtualDirect undoScreen = new VirtualDirect();
  static PalettePanel palette;

  // Constants to define which graphics operation the user has selected
  final static int PICK = 32;
  final static int DOT = 0;
  final static int LINE = 1;
  final static int FILL = 2;
  final static int MOVE  = 4;
  final static int SPRAY = 5;
  final static int RUBBER = 6;
  final static int RECTANGLE = 7;
  final static int ELLIPSE = 8;

  // MODE indicates the currently selected graphics operation
  static int MODE = DOT;

  int OLDCOLOR;
  int X;
  int Y;


  static boolean TRACE = true;   // pen up/down
  static int SQUARE = 0;
  static int WIDTH = 0;   // Taille image relle
  static int HEIGHT = 0;

  // pixel stores the mouse pointer position
  //static Point pixel = null;

  static int x1, y1;
  static int x2, y2;
  static int colorInt = 0;

  PopupMenu mousemenu = new PopupMenu();

  RealView realView = null;

  ActionListener actionListener = null;
//  private Key key = null;

  static void error(String s) { System.out.println(s); }

// Voir MouseMotionListener, le paramtre tait this seul

  public Painter()
   {
  	 BuildMouseMenus();
//   addMouseListener(this);  // Dans build
     addMouseMotionListener(this);
     addKeyListener(this);
     addFocusListener(this);
	 //pixel = new Point(0, 0);

	 OLDCOLOR = 0;
	 X = 0;
	 Y = 0;

   };

   Panel panel;

   public Painter(Panel p, VirtualBig vs, VirtualDirect rs)
   {
     this();
     panel = p;
     setVirtualScreen(vs, rs); // appelle setSize()
   };



   public void setSize(int w, int h)
   {
    Dimension d = panel.getSize();
    int pw = d.width;
    int ph = d.height;

    setBounds((pw - w) / 2, (ph - h) / 2, w, h);
   }

  /**
     Met  jour le pointeur d'cran virtuel
  */
  void setVirtualScreen(VirtualBig vs, VirtualDirect rs)
  {
   //error("Painter - w = " + vs.width + " h " + vs.height);

   bigScreen = vs;
   realScreen = rs;

   SQUARE = vs.SQUARE;
   WIDTH  = rs.width;
   HEIGHT = rs.height;

   //error(" w = " + WIDTH + " h " + HEIGHT);
   setSize(bigScreen.width, bigScreen.height);
  }

  /** Spcifie image relle associe */
  void setRealView(RealView p)
  {
   realView = p;
  }

  /** Spcifie la palette */
  void setPalette(PalettePanel pal)
  {
    palette = pal;
  }


  /** Affecte indicateur de position */

  Label labelX;
  Label labelY;

  void setPosIndicators(Label lx, Label ly)
  {
   labelX = lx;
   labelY = ly;
  }

  public void BuildMouseMenus()
   {
    Font menuFont = new Font("Dialog", Font.PLAIN, 14);

	MenuItem pickItem = new MenuItem("Pick Color");
	pickItem.addActionListener(this);
	pickItem.setFont(menuFont);
    mousemenu.add(pickItem);

	MenuItem moveItem = new MenuItem("Move Cursor");
	moveItem.setFont(menuFont);
	moveItem.addActionListener(this);
    mousemenu.add(moveItem);

    MenuItem fillItem = new MenuItem( "Fill" );
    fillItem.addActionListener(this);
	fillItem.setFont(menuFont);
    mousemenu.add(fillItem);

    MenuItem shadeItem = new MenuItem( "Shade" );
    shadeItem.addActionListener(this);
	shadeItem.setFont(menuFont);
    mousemenu.add(shadeItem);

    mousemenu.addSeparator();

    MenuItem dotItem  = new MenuItem( "Dot Mode" );
    dotItem.addActionListener(this);
	dotItem.setFont(menuFont);
    mousemenu.add(dotItem);

    MenuItem rubberItem  = new MenuItem( "Rubber Mode" );
    rubberItem.addActionListener(this);
	rubberItem.setFont(menuFont);
    mousemenu.add(rubberItem);


    MenuItem sprayItem  = new MenuItem( "Spray Mode" );
    sprayItem.addActionListener(this);
	sprayItem.setFont(menuFont);
    mousemenu.add(sprayItem);

    MenuItem lineItem = new MenuItem( "Line" );
    lineItem.addActionListener(this);
	lineItem.setFont(menuFont);
    mousemenu.add(lineItem);

    MenuItem rItem = new MenuItem( "Rectangle" );
    rItem.addActionListener(this);
	rItem.setFont(menuFont);
    mousemenu.add(rItem);

    MenuItem eItem = new MenuItem( "Ellipse" );
    eItem.addActionListener(this);
	eItem.setFont(menuFont);
    mousemenu.add(eItem);


//    Menu ToolMenu = new Menu("Tool");

//    mousemenu.add(ToolMenu);

    // add mousemenu
	this.add(mousemenu);
	addMouseListener(this);

  }



   /**
       Paint
   */

   public void update(Graphics g) { paint(g); }

   public void paint(Graphics g)
   {
    Image img = bigScreen.getImage();

    MediaTracker mt = new MediaTracker(this);
    mt.addImage(img, 0);
    try { mt.waitForAll(); } catch(InterruptedException e) { }

   //error("Painter draw - image w " + img.getWidth(null));
   //error("Painter draw - image h " + img.getHeight(null));

    try { g.drawImage(img, 0, 0, getBackground(), this); }
    catch(Exception e)
    {
     error("Painter - " + e.toString());
     error("Image size: " + img.getWidth(null) + " " + img.getHeight(null));
    }

  }


//------------------------------ Dessin


  // Change la couleur courante
  static public void setColor(int c)
  {
    realScreen.COLOR = c;
    bigScreen.COLOR  = c;
    //System.out.println("painter set to " + c);
  }

  // Change la couleur courante
  static public void setColor(Color color)
  {
    setColor(PalettePanel.colorToInt(color));
  }


  /**
     retourne l'inverse d'une couleur
  */
  static public int invert(int couleur)
  {
   /*
   int r = couleur & 0x00FF0000;
   int g = couleur & 0x0000FF00;
   int b = couleur & 0x000000FF;
   */
   return (0xFFFFFFFF - couleur) | 0xFF000000;

  }



 public void unCursor()
 {
   realScreen.COLOR = OLDCOLOR;
   bigScreen.COLOR = OLDCOLOR;
   setPixel();
 }

 public void cursor()
 {
   OLDCOLOR = realScreen.getPixel(X, Y);
   realScreen.COLOR = invert(OLDCOLOR);
   bigScreen.COLOR = realScreen.COLOR;
   setPixel();
 }


//-------------------------------- Fonctions de dessin

  void doImage()
  {
   //System.out.println("painter doImage - " + realScreen.width + "x" + realScreen.height);
   undoScreen = new VirtualDirect(realScreen.width, realScreen.height);
   realScreen.copy(undoScreen);
  }

  void undoImage()
  {
   undoScreen.copy(realScreen);

  }
  /**  Prend une couleur dans un pixel */

  int PICKCOLOR;

  public int pickColor(int x, int y) { return realScreen.getPixel(x, y); }
  public int pickColor()
  {
    int x = mx / SQUARE;
    int y = my / SQUARE;
    y = (HEIGHT - 1) - y;

    //error("xy " + x + " " + y);
    return pickColor(x, y);
 }

  /** Dot */

  public void setPixel() { setPixel(X, Y); }
  public void setPixel(int x, int y)
  {
    int color = PalettePanel.penInt;
    realScreen.pixel(x ,y, color);
    bigScreen.pixel(x, y, color);
    realView.repaint();
    repaint();
  }

  public void setPixel(int x, int y, int color)
  {
    realScreen.pixel(x ,y, color);
    bigScreen.pixel(x, y, color);
    realView.repaint();
    repaint();
  }


  void update()
  {
   bigScreen.update();
   realView.repaint();
   repaint();

   setColor(PalettePanel.penInt);
  }

  /** Pot de peinture */

  public void fill()
  {
   doImage();
   setXY();
   realScreen.fill(X, Y);
   update();
  }


 /**
    Spray
    Undo seulement pour une opration simple (non drague)
 */

 void spray() { spray(X, Y); }
 void spray(int x, int y)
 {
  realScreen.spray(x, y);
  update();
 }

  /** shade */

 void shade()
  {
   doImage();
   setXY();
   realScreen.shade(pickColor(X, Y), 1, palette);
   update();
  }

  /** Inversion */

  void flipV() { realScreen.flipV(); update(); }
  void flipH() { realScreen.flipH(); update(); }

  /** Mirror */

  void mirrorH() { doImage(); realScreen.mirrorH(); update(); }
  void mirrorV() { doImage(); realScreen.mirrorV(); update(); }


  /** Edition */

  void insertV() { doImage(); realScreen.insertV(Y); update(); }
  void deleteV() { doImage(); realScreen.deleteV(Y); update(); }
  void insertH() { doImage(); realScreen.insertH(X); update(); }
  void deleteH() { doImage(); realScreen.deleteH(X); update(); }


  public void c2c()
  {
    doImage();
    int old   = PalettePanel.selectedInt;
    int next  = PalettePanel.colorToInt(PalettePanel.replaceColor);
    realScreen.colorToColor(old, next);
    update();
  }


  public void scrollUp()    { realScreen.scrollUp();    update(); }
  public void scrollLeft()  { realScreen.scrollLeft();  update(); }
  public void scrollRight() { realScreen.scrollRight(); update(); }
  public void scrollDown()  { realScreen.scrollDown();  update(); }


  public void rotate()
  {

  }


  public void merge(Image vImage)
  {
   doImage();

   VirtualImage vi = new VirtualImage();
   vi.load(vImage);

   int transp = PalettePanel.transparentInt;

   realScreen.merge(vi.getPixels(), vi.width, vi.height, transp);
   update();
  }


 void setXY()
 {
  X = mx / SQUARE;
  Y = my / SQUARE;
  Y = (HEIGHT - 1) - Y;
 }

//------------------------------ Pop up


  static VirtualBig bClone = null;
  static VirtualDirect tempScreen;

  void doShape()
  {
    tempScreen = new VirtualDirect(realScreen.width, realScreen.height);
    realScreen.copy(tempScreen);

    //int sq = bigScreen.SQUARE;
    if(bClone == null)
     bClone = new VirtualBig(bigScreen.width, bigScreen.height);

    bigScreen.copy(bClone);
  }

  void undoShape()
  {
    bClone.copy(bigScreen);
    tempScreen.copy(realScreen);
  }


  void line()
  {
    undoShape();
    realScreen.line(x1 ,y1, X, Y, colorInt);
    bigScreen.line(x1, y1, X, Y, colorInt);

    realView.repaint();
    repaint();
  }

  void ellipse()
  {
    undoShape();
    int r = X - x1;
    int r2 = Y - y1;
    realScreen.ellipse(x1 ,y1, r, r2, colorInt);
    bigScreen.ellipse(x1, y1, r, r2, colorInt);

    realView.repaint();
    repaint();
  }


  void rectangle()
  {
    undoShape();
    realScreen.rectangle(x1 ,y1, X, Y, colorInt);
    bigScreen.rectangle(x1, y1, X, Y, colorInt);

    realView.repaint();
    repaint();
  }


  void lineMode()
  {
    doImage();
    MODE = LINE;
    doShape(); // sauve l'cran initial
    colorInt = PalettePanel.penInt;
    x1 = X;
    y1 = Y;
    X = x1 + 10;
    Y = y1 + 10;

    line();
  }

  void ellipseMode()
  {
    doImage();
    MODE = ELLIPSE;
    doShape(); // sauve l'cran initial
    colorInt = PalettePanel.penInt;
    x1 = X;
    y1 = Y;
    X = X + 10;
    Y = Y + 10;

    ellipse();
  }


  void rectangleMode()
  {
    doImage();
    MODE = RECTANGLE;
    doShape(); // sauve l'cran initial
    colorInt = PalettePanel.penInt;
    x1 = X;
    y1 = Y;
    X = x1 + 10;
    Y = y1 + 10;
    rectangle();
  }

//---------------------------------------- Mouse

  int mx;
  int my;

  public void mouseMoved(MouseEvent e)
  {
    mx = e.getX();
    my = e.getY();

    int x = mx / SQUARE;
    int y = (HEIGHT - 1) - (my / SQUARE);

    labelX.setText(Integer.toString(x));
    labelY.setText(Integer.toString(y));
  }


  int oldX = -1;
  int oldY = -1;

  public void mouseDragged(MouseEvent e)
  {
    e.consume();

    if (e.isPopupTrigger())  return;

    mx = e.getX();
    my = e.getY();

    X = mx / SQUARE;
    Y = my / SQUARE;

    if(X < 0) return;
    if(Y < 0) return;
    if(X >= WIDTH) return;
    if(Y >= HEIGHT) return;

    Y = (HEIGHT - 1) - Y;

    switch (MODE)
    {
      case DOT:
               if((oldX != X) || (oldY != Y))
                {
                 oldX = X;
                 oldY = Y;
                 setPixel(X, Y);
                }
               break;

      case RUBBER:
                if((oldX != X) || (oldY != Y))
                {
                 oldX = X;
                 oldY = Y;
                 setPixel(X, Y, PalettePanel.transparentIndex);
                }
                break;

      case SPRAY:     spray(); break;
      case LINE:      line();   break;
      case RECTANGLE: rectangle(); break;
      case ELLIPSE:   ellipse(); break;

      default:    break;
     }

  }


  public void mousePressed(MouseEvent e)
  {
     if (e.isPopupTrigger())                 // right button
     {
   		this.mousemenu.show(this, mx, my);
   		return;
	 }

  }


  public void mouseReleased(MouseEvent e)
   {
     e.consume();

     mx = e.getX();
     my = e.getY();

     X = mx / SQUARE;
     Y = my / SQUARE;
     Y = (HEIGHT - 1) - Y;

     if (e.isPopupTrigger())                 // right button
     {
   		this.mousemenu.show(this, mx, my);
   		return;
	 }


    // left button
    switch (MODE)
    {
     case DOT:
                  doImage(); setPixel(X,  Y);
                  break;
     case RUBBER: doImage(); setPixel(X, Y, PalettePanel.transparentIndex);
                  break;
     case SPRAY:  doImage(); spray();
                  break;
     case RECTANGLE:  // retour normale
                  MODE = DOT;
                  break;
     case LINE:
                  MODE = DOT;
                  break;
     case ELLIPSE:MODE = DOT;
                  break;
     default:     break;
    }

  }



    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) {	requestFocus();}
    public void mouseClicked(MouseEvent e) {}

    // implement actions for each menu item
    public void actionPerformed(ActionEvent e)
    {
        String ac = e.getActionCommand();

        System.gc();
        if (ac.equals("Pick Color"))
        {
          PICKCOLOR = pickColor();
          sendAction("pick");
        }
        else if (ac.equals("Move Cursor"))  setXY();
        else if (ac.equals("Fill"))   fill();
        else if (ac.equals("Shade"))  shade();
        else if (ac.equals("Dot Mode"))
        {
         MODE = DOT;
         setColor(PalettePanel.penInt);
         //System.out.println("paint set color " + PalettePanel.penInt + " "+  PalettePanel.penIndex);
        }
        else if (ac.equals("Rubber Mode")) MODE = RUBBER;
        else if (ac.equals("Line"))      lineMode();
        else if (ac.equals("Rectangle")) rectangleMode();
        else if (ac.equals("Ellipse"))   ellipseMode();
        else if (ac.equals("Spray Mode"))  MODE = SPRAY;
   }


//------------------------------------ Focus

   public void focusGained(FocusEvent e) { }
   public void focusLost(FocusEvent e) { }


//------------------------------------ Keyboard


    public void keyReleased(KeyEvent e) { };
    public void keyTyped(KeyEvent e) { };

   	/** Key Pressed events */
	public void keyPressed(KeyEvent e)
	{
       int code = e.getKeyCode();

       //error("code " + code + " " + e.toString());

       if (e.isMetaDown() || e.isAltDown())  return;

       // Avec CTRL
       if(e.isControlDown())
       {
         switch(code)
         {
          case KeyEvent.VK_UP:
	                break;
          case KeyEvent.VK_DOWN:
          	        break;
          case KeyEvent.VK_Z:
                    //error("painter undo");
                    sendAction("undo");
                    break;
         }
         return;
       }

       unCursor();
       // Sans CTRL
       switch(code)
        {
          case KeyEvent.VK_ENTER:  sendAction("Enter"); requestFocus(); break;
          case KeyEvent.VK_DELETE: sendAction("Delete"); requestFocus(); break;
          case KeyEvent.VK_ESCAPE: sendAction("Esc"); break;

          case KeyEvent.VK_HOME:
                    X = 0; Y = 0;
                	break;
          case KeyEvent.VK_END:

	                break;
          case KeyEvent.VK_UP:
                    Y++;
	                break;
          case KeyEvent.VK_LEFT:
                    X--;
                    break;
          case KeyEvent.VK_RIGHT:
                    X++;
                    break;
          case KeyEvent.VK_DOWN:
                    Y--;
          	        break;

          case KeyEvent.VK_SPACE: TRACE = !TRACE;
                    return;

	      default:  //Disp.it("default " + code);
                    break;
         }

        cursor();
      }

/**
    Sends an action performed event to any action listeners
 */
  protected void sendAction(String command)
   {
    //error("sendaction " + command);
	if (actionListener != null)
	 actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, new String(command)));
   }

  /**
    Adds the specified action listener to receive action events
   */
  public void addActionListener(ActionListener l)
   {
 	actionListener = AWTEventMulticaster.add(actionListener, l);
   }


} // class


