/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.image.*;


public class Plasma extends Canvas
{
  PlasmaThread thrPlasma;
  Image imagePlasma;
  Image imageOffScreen;

  int width = 420;
  int height = 300;

  // Image seule (1) ou plusieurs (autres valeurs)
  private final static int TILEX = 1;
  private final static int TILEY = 1;

  private int destPixels[];
  private int countPixels;

  PaletteGreen palette =  new PaletteGreen();
  //PaletteShaded palette  = new PaletteShaded();
//  PalettePlasma palette = new PalettePlasma();


  public Plasma()  {  }

  void debug(String s) { System.out.println(s); }



 ///////////////// B A C K G R O U N D ///////////


  /** Initialisation section */

  boolean initOk = false;

  public void plasmaInit()
   {
  	Dimension dim = getSize();
	  init(dim.width, dim.height);
   }

  public void init(int w, int h)
  {
    if(initOk) return;
    width  = w / TILEX;
    height = h / TILEY;

    drawBackground();
    initOk = true;
  }


  private void drawBackground()
  {
    // Tableau des pixels cr et mis  0
	countPixels = width * height;
	destPixels = new int[countPixels];
	for (int i = 0; i < countPixels; i++)	destPixels[i] = 0;


	destPixels[0] = 128;
	destPixels[width - 1] = 128;
	destPixels[(width * (height - 1))] = 128;
	destPixels[(width * (height - 1)) + width - 1] = 128;

	calcPoints(0, 0,
        width - 1, height - 1,
	destPixels[0], destPixels[width - 1],
	destPixels[width * (height - 1)], destPixels[(height * width) - 1]);
  }

private void calcPoints(int x1, int y1, int x4, int y4, int l1, int l2, int l3, int l4)
  {
	if (((x1 + 1) >= x4) && ((y1 + 1) >= y4))	return;

	int w1 = (x4 - x1) >> 1;
	int h1 = (y4 - y1) >> 1;

	int m1;
	int m2;
	int m3;
	int m4;
	int m5;

	// Draw point 1
	if (destPixels[x1 + w1 + (width * y1)] == 0)
        {
		m1 = deviate(l1, l2, w1);
		destPixels[x1 + w1 + (width * y1)] = m1;
	}
	else
		m1 = destPixels[x1 + w1 + (width * y1)];

	// Draw point 2
	if (destPixels[x1 + (width * (y1 + h1))] == 0)
        {
		m2 = deviate(l1, l3, h1);
		destPixels[x1 + (width * (y1 + h1))] = m2;
	}
	else
		m2 = destPixels[x1 + (width * (y1 + h1))];

	// Draw point 4
	if (destPixels[x4 + (width * (y1 + h1))] == 0)
         {
		m3 = deviate(l2, l4, h1);
		destPixels[x4 + (width * (y1 + h1))] = m3;
	 }
	 else
		m3 = destPixels[x4 + (width * (y1 + h1))];

	// Draw point 4
	if (destPixels[x1 + w1 + (width * (y4))] == 0)
         {
		m4 = deviate(l3, l4, w1);
		destPixels[x1 + w1 + (width * (y4))] = m4;
	 }
	else
		m4 = destPixels[x1 + w1 + (width * (y4))];

	// Draw point 5
	if (destPixels[x1 + w1 + (width * (y1 + h1))] == 0)
        {
		m5 = deviate(m1, m4, h1);
		destPixels[x1 + w1 + (width * (y1 + h1))] = m5;
	}
	else
		m5 = destPixels[x1 + w1 + (width * (y4))];

		calcPoints(x1, y1, x1 + w1, y1 + h1, l1, m1, m2, m5);
		calcPoints(x1 + w1, y1, x4, y1 + h1, m1, l2, m5, m3);
		calcPoints(x1, y1 + h1, x1 + w1, y4, m2, m5, l3, m4);
		calcPoints(x1 + w1, y1 + h1, x4, y4, m5, m3, m4, l4);
	}

  private int deviate(int l1, int l2, int dist)
   {
	int diff = Math.abs(l2 - l1);
	int avg = (l1 + l2 ) / 2;
	int offset = -dist * avg  / 255;

	diff = (int)(dist * Math.random()) + offset;

	int l3 = avg + diff;

	if (l3 < 0)			   l3 = -l3;
	else if (l3 > 255) l3 = 510 - l3;

	return l3;
   }

  /** Dessin de l'image */

public void update(Graphics g) { paint(g);}

public void paint(Graphics g)
  {
    try
     {

      makeScreen();

      if ((imageOffScreen == null) || (palette == null))	return;


      for (int i = 0; i < TILEX; i++)
	for (int j = 0; j < TILEY; j++)
	  g.drawImage(imageOffScreen, i * width, j * height, this);

      /*
		  palette.rotateRed(5);
		  palette.rotateGreen(2);
		  palette.rotateBlue(3);
      */
      palette.rotate(4);
      palette.setColorModel();

      Thread.sleep(50);

     }
      catch (InterruptedException e)
      {
	System.out.println("InterruptedException: " + e);
	isRunning = false;
      }
   }




  /**
     Cre l'image plasma
  */

  void makePlasma()
  {
   try
     {
       imagePlasma = createImage(
       new MemoryImageSource(width, height, palette.getColorModel(),
			destPixels, 0, width));

     MediaTracker o = new MediaTracker(this);
     try {
      o.addImage(imagePlasma, 0);
      o.waitForAll();
     } catch(Exception e) {};

		}
    catch (Exception e)
     {
	System.out.println(e);
	isRunning = false;
     }

   }

  /**
     Cre le buffer off screen

     Place une image source dessus
     Place le texte dessus
  */

  void makeScreen()
  {
    try
     {

     imageOffScreen = this.createImage(width, height);

     Graphics g = imageOffScreen.getGraphics();

     makePlasma();
     g.drawImage(imagePlasma, 0, 0, this);

      }
    catch (Exception e)
     {
	System.out.println(e);
	isRunning = false;
      }

   }

  //////////// T H R E A D ///////////////////


  class PlasmaThread extends Thread
  {

   public void run()
   {
    plasmaInit();
    isRunning = true;
    while(isRunning) repaint();
   }
 }


  public void start()
   {
     if (thrPlasma == null)
     {
	isRunning = true;
	thrPlasma = new PlasmaThread();
	thrPlasma.start();
     }

    isRunning = true;
   }

  public void stop()
   {
    isRunning = false;
    if(thrPlasma != null)
     {
      thrPlasma.interrupt();
      thrPlasma = null;
     }
   }

  private boolean isRunning = false;

}

