/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.*;
import common.*;

/**
   Button which pops up when the mouse scrolls over it.
   Displays a image, a textual label or both.

   Ne doit pas requrir le focus car il opre sur des
   listes dfinies par le focus
*/

public class DButton extends Component
{
   String label;
   String imageURL;              // filename URL for image
   Image image;                  // image
   Image imageNormal;
   Image imageGrayed;
   boolean inButton = false;     // has the mouse entered the button
   boolean pressed  = false;     // has the user pressed the mouse button

   boolean bFixed = false;

   static int DefaultWidth;     // Taille pour une srie de boutons
   static int DefaultHeight;
   static Font DefaultFont = new java.awt.Font("Dialog", Font.PLAIN, 14);

   int Width;
   int Height;
   int ImageZoneWidth = 0;      // Rpartition entre image et label
   int ImageZoneHeight = 0;


   final static int NONE = 0;
   final static int EAST = 1;
   final static int SOUTH = 2;
   final static int WEST = 3;
   final static int NORTH = 4;
   final static int CENTER = 5;
   int labelPosition = CENTER;
   static boolean ADJUSTSIZE = false;  // Ajuste selon intrieur
   static boolean ADJUSTMAX = false;   // Agrandit seulement (dfaut donn)

   String actionCommand;
//   private Action	action		= null;
   private DMouse	mouse		= null;
   ActionListener actionListener = null;

   public Timer      notifyTimer;
   public int       notifyDelay;
   public boolean   notifyWhilePressed;
   transient public boolean   running;
   transient public boolean   notified;
   transient public boolean   released;

   Tip tip = null;

   /** DButton empty */
   public DButton( )
   {
    super();
    Width  = DefaultWidth;
    Height = DefaultHeight;

    pressed				= false;
    released			= true;
    notifyWhilePressed	= false;
    running				= false;
    notified			= false;
    notifyTimer			= null;
    notifyDelay			= 1000;
    setFont(DefaultFont);
   }

   public DButton(boolean b)
   {
    this();
    bFixed = b;
   }

   /** Constructs a DButton with an image  */
   public DButton (Image img )
   {
     this();
     this.imageNormal = img;
     this.imageGrayed = img;
     this.setup();    // Appelle AdjustSize

     //System.out.println("dbutton " + Width + "x" + Height);
   }

   /** Constructs a DButton with two images  */
   public DButton (Image img, Image gray)
   {
     this();
     this.imageNormal = img;
     this.imageGrayed = gray;
     this.setup();    // Appelle AdjustSize
   }


   /** DButton with both String and an Image  */
   public DButton (Image img, String lab)
   {
       this();
       label = lab;
       labelPosition = EAST;
       this.imageNormal = img;
       this.imageGrayed = img;
       this.setup();

   }



   public DButton (String img, String lab, int labpos)
   {
       this();
       this.imageNormal = Toolkit.getDefaultToolkit().getImage(img);
       this.imageGrayed = imageNormal;
       label = lab;
       labelPosition = labpos;
       this.setup();
   }


   /** DButton with both Label and an Image as string  */
   public DButton (String img, String lab)
   {
       this(img, lab, EAST);
   }


   /** Bouton selon nom fichier */
   public DButton (String nomimage)
   {
     this();
     this.imageNormal = Toolkit.getDefaultToolkit().getImage(nomimage);
     this.imageGrayed = imageNormal;
     this.setup();
   }


   /** Bouton classique indique si toujours dessin */
   public DButton (String lab, boolean fixed)
   {
      this();
      label = lab;
      labelPosition = CENTER;
      bFixed = fixed;
      //System.out.println(lab + " font " + DefaultFont.toString());
   }

   public DButton (Image img, String lab, boolean fixed)
   {
      this(img, lab);
      labelPosition = EAST;
      bFixed = fixed;
   }


   /** Set label position */
   public void setLabelPosition(int p)
    {
      if(p >= NONE)
       if(p <= CENTER)
        labelPosition = p;
    }

    /** Change font par dfaut */

   static public void setDefaultFont(Font f) { DefaultFont = f;  }

    /**
     * Sets the Image portion of this DButton's label.
     * Ensures image is loaded before using it.
     */

    public void setImage( Image img )
    {
        this.image = img;
        if(image.getWidth(this) == -1  ||  image.getHeight(this) == -1 )
            this.ensureImageLoaded( image );

        repaint();
    }


   public void setLabel(String lab)
   {
     setLabel(lab, false);
   }

   public void setLabel(String lab, boolean resize)
   {
     label = lab;
     if (resize)
     {
      int w = getFontMetrics(getFont()).stringWidth(label);
      setSize(w + 8, Height);
     }
     repaint();
   }

   /**
     Selon taille du texte et image avec bords
     Retourne la Dimension needed
   */

   private void neededSize(Graphics g)
    {
      int wl = 0;   // rectangle texte
      int hl = 0;
      int wi = 0;   // rectangle image
      int hi = 0;

      if(!ADJUSTSIZE) return;

       // Calculate image dimensions

      if( this.image != null )
       {
         int imageWidth = image.getWidth( this ) + 4;
         wi = Width > imageWidth ? Width : imageWidth;

         int imageHeight = image.getHeight( this ) + 4;
         hi = Height > imageHeight ? Height : imageHeight;
       }


      int nw = wi;   // largeur combine, image par defaut
      int nh = hi;   // hauteur combine

      if( label != null )
       {
        // Calculate label dimensions
        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        hl = fm.getHeight() + 4;
        wl = fm.stringWidth(label) + 6;

        if(labelPosition == EAST)
         {
          nw = wl + wi;
          nh = hi > hl ? hi : hl;
         }
        else
        if(labelPosition == SOUTH)
         {
          nw = wi > wl ? wi : wl;
          nh = hi + hl;
         }

       }

      // Taille combine si les deux sont l, ou le max
      if(ADJUSTMAX)
      {
       if(nw > Width)  Width = nw;
       if(nh > Height) Height = nh;
      }
      else
      {
       Width = nw;
       Height = nh;
      }

   } // real size


   // partage

   // selon dimensions image et dimensions texte
   // et selon taille du composant


    private void CalculateZones(int iw, int ih, int lw, int lh)
    {
     ImageZoneWidth = 0;
     ImageZoneHeight = 0;

     if(labelPosition == EAST)
        ImageZoneWidth = Width * iw / (iw + lw + 8 + 1);
     else
     if(labelPosition == SOUTH)
        ImageZoneHeight = Height * ih / (ih + lh + 8 + 1);

    } // CalculateZones


   // Appele par addNotify
   // quand la taille est non initialise par set Size et vaut donc 0.
   // Il semble que la fonction opre  retardement ds qu'un contexte
   //  graphique est obtenu, donc quand on paint le bouton

   public java.awt.Dimension adjustSize( Graphics g )
   {
     if(g != null ) neededSize(g);  // Evaluate needed dimensions
     return new java.awt.Dimension( Width, Height );
   }

   /** setSize
       Surcharge la fonction pour avoir la taille dans les variables
       de la classe
       Et ajuste si la fonction est demande
   */


   public void setSize(int w, int h)
   {
     Width = w;
     Height = h;
     if(ADJUSTSIZE) neededSize(this.getGraphics());
     super.setSize(Width, Height);
   }

   public void setBounds(int x, int y, int w, int h)
   {
     Width = w;
     Height = h;
     if(ADJUSTSIZE) neededSize(this.getGraphics());

     super.setBounds(x, y, w, h);
   }

    /**
     * Paints label over image, handles popping up, depressing, orientation and title position
     * issues.  Will not draw if this DButton has not been added to the screen.
     */
   public void update(Graphics g) { paint(g); }

   public void paint(Graphics g)
   {
     boolean labelpresent = false;
     int lh;
     int lw;

     //Disp.it("paint button");

     if( isShowing() )
      {
        //Disp.it("showing");
        if(!isEnabled())
         {
           inButton = false;
           pressed = false;
         }

        java.awt.Dimension mySize = getSize();  // Taille du bouton
        int w = mySize.width - 2;
        int h = mySize.height - 2;
        Width = w;          // Mise  jour car AdjustSize est  retardement
        Height = h;

        lw = 0;   // a cause message
        lh = 0;

        // Color using desired background color

        Color BorderLight = null;
        Color BorderDark= null;
        Color BackColor = getBackground();
        Color pressedBackColor	= Effect.darken(BackColor, (double) 1 / 10);

        g.setColor( BackColor );

        boolean drawBorders = false;
        int padding = 0;

        if( inButton  &&  pressed )
         {
          // Color for when mouse is over this and button has been depressed
          BorderLight = BackColor.darker();
          BorderDark = BackColor.brighter();
		  //BorderLight = ColorUtils.darken(BackColor,	(double) 1 / 2);
		  //BorderDark  = ColorUtils.lighten(BackColor,	(double) 1 / 2);

          g.setColor( pressedBackColor );
          drawBorders = true;
          //padding = 1;
          padding = 0;
         }
        else
        if( inButton || bFixed)
         {
          // Color for when mouse is over this button but not depressed
          BorderLight = BackColor.brighter();
          BorderDark =  BackColor.darker();
		  //BorderLight = ColorUtils.lighten(BackColor,	(double) 1 / 2);
		  //BorderDark  = ColorUtils.darken(BackColor,	(double) 1 / 2);

          drawBorders = true;
          padding = -1;
         }


        g.fillRect( 1, 1, w - 2, h - 2 );

        // Dessine le bord, quand la souris est sur le bouton
        // ou quand il est presse

        if( drawBorders )
         {
          int x = 1;
          int y = 1;
          int xr = x + w - 1;
          int yb = y + h - 1;

         //  Disp.it("draw border");

       // Je met un cadre noir autour
          g.setColor(Color.black);
          g.drawRect(x - 1, y - 1, xr + 1, yb + 1 );

          g.setColor( BorderLight );
          g.drawLine( x, y, xr, y );
          g.drawLine( x, y , x, yb);
          g.setColor( BorderDark );
          g.drawLine( x, yb, xr, yb );
          g.drawLine( xr, y, xr, yb );
        } // if drawborders

      if( label != null  &&  label.length() > 0 )
       {
         labelpresent = true;
         g.setFont(getFont());
         FontMetrics fm = g.getFontMetrics();
         lh = fm.getHeight();
         lw = fm.stringWidth(label) + 6;
       }


      // Draw image

      int xImg = 0;      // Dcalage image
      int yImg = 0;

      if(isEnabled()) image = imageNormal;
      else image = imageGrayed;

      //if (image == null) Disp.it("image null");

      // Center and draw image
      if( image != null )
      {
       //Disp.it("center draw");
       int iw = image.getWidth( this )  + 4;
       int ih = image.getHeight( this ) + 4;

       if(labelpresent)  // label et image presents
       {
         CalculateZones(iw, ih, lw, lh);   // partage

         if(labelPosition == EAST)
         {
           xImg = (ImageZoneWidth - iw) / 2 + 4;
           yImg = (h - ih) / 2 + 2;
         }
         else
         if(labelPosition == SOUTH)
         {
           xImg = (w - iw) / 2 + 4;
           yImg = (ImageZoneHeight - ih) / 2 + 2;
         }
       }
       else // Image seule
       {
        xImg = (w - iw) / 2 + 4;
        yImg = (h - ih) / 2 + 4;
       }

       if(isEnabled() || labelpresent)
          g.drawImage( image, xImg + padding, yImg + padding, this );
       else // image seule
       {
         g.drawImage( image,
                      xImg * 4 / 3 + padding - 1,
                      yImg * 4 / 3 + padding - 1,
                      iw * 3 / 4,
                      ih * 3 / 4,
                      this );

       }
      } // if image

      // Placer le text selon position

      if(labelpresent)
      {

       int xLab = 0;
       int yLab = 0;
       int YLoff = 14;

       if(labelPosition == SOUTH)
       {
        xLab = (w - lw) / 2 + 3;
        yLab = ImageZoneHeight + ((h - ImageZoneHeight) - lh) / 2 + YLoff + 2;
       }
       else
       {
        yLab = (h - lh) / 2 + YLoff+ 2;

        if(labelPosition == EAST)
          xLab = ImageZoneWidth + ((w - ImageZoneWidth) - lw) / 2 + 3;
        else
          if(labelPosition == CENTER)
            xLab = (w - lw) / 2 + 4;
       }

      // Draw title in foreground color

      if(isEnabled())
      {
       g.setColor( this.getForeground() );
       g.drawString(label, xLab + padding, yLab + padding);
      }
      else
      {
       g.setColor( Color.white );
       g.drawString(label, xLab + padding + 1, yLab + padding + 1);
       g.setColor( this.getBackground().darker() );
       g.drawString(label, xLab + padding, yLab + padding);
      }

     }
   }


 }


    /**
     * Makes sure that any image associated with this DButton is loaded
     * so that its width and height have valid values.
     */

    private final void ensureImageLoaded(Image image)
    {
       MediaTracker tracker = new MediaTracker( this );
       tracker.addImage( image, 0 );

       try { tracker.waitForAll(); }
       catch(InterruptedException e) { }
    }


    /**
     * Do initialization with image before calculating size of DButton.
    */
    private final void setup()
    {
      if(image != null)
       if(image.getWidth(this)==-1  ||  image.getHeight(this)==-1)
          this.ensureImageLoaded( image );
    }


  // Dfinit la taille d'une srie de boutons

     static public void setDefaultSize(int w, int h)
      {
       DefaultWidth = w;
       DefaultHeight = h;
      }


   /**
      Handle any mouse activity on this DButton, especially processing
      mouse entered, mouse exited, mouse pressed, and mouse released.
      Pass all other activity up the inheritance chain.
    */

	class DMouse extends java.awt.event.MouseAdapter implements java.io.Serializable
	{
		public void mousePressed(MouseEvent e)
		{
	        //requestFocus();
	        notified = false;
	        if (notifyTimer != null && notifyWhilePressed && !running)
	        {
	          running     = true;
	          notifyTimer.start();
	        }

	        pressed           = true;
	        released          = false;

   			if(tip != null) tip.hideIt();
	        repaint();
		}

	    public void mouseReleased(MouseEvent e)
	    {
	        if (notifyTimer != null && running)
	        {
	         running = false;
	         notifyTimer.stop();
	        }

	        if (pressed)
	        {
              pressed = false;

	          if (!notifyWhilePressed || !notified)
	           {
	             sourceActionEvent();
	           }
	        }

	        released = true;
			if(tip != null) tip.hideIt();
	        if(inButton) repaint();
	    }

		/** Handles DMouse Entered events */
		public void mouseEntered(MouseEvent e)
		{
	        inButton = true;

	        if (!released)
	        {
		      pressed = true;

			  if(notifyTimer != null && notifyWhilePressed && !running)
			   {
		         running = true;
		         notifyTimer.start();
			   }

			}

		    repaint();

	        if(tip != null)  tip.showIt();
		}

		public void mouseExited(MouseEvent e)
		{
	        inButton = false;
            pressed = false;

	        if (notifyTimer != null && running)
	        {
	            running = false;
	            notifyTimer.stop();
	        }

			if(tip != null) tip.hideIt();

  			repaint();
		}
	}


    /**
     * Fire an action event to the listeners.
     */
        public void sourceActionEvent()
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCommand));
	}

    /**
     * Adds the specified action listener to receive action events
     * from this button.
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}
    /**
     * Removes the specified action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}


   // DBaseComponent
    public void addNotify()
    {
       super.addNotify();

       java.awt.Dimension d = getSize();
       if( d.width == 0  ||  d.height == 0 )
          setSize( this.adjustSize( this.getGraphics() ) );
	   if (mouse == null)
		{
		 mouse = new DMouse();
		 addMouseListener(mouse);
		}

    }


    public void removeNotify()
    {
   	  if (mouse != null)
		{
			removeMouseListener(mouse);
			mouse = null;
		}

	   super.removeNotify();
    }


    /** Overridden to return the preferred size of the component  */
    public java.awt.Dimension getMinimumSize()
    {
        return getPreferredSize();
    }


    /**
       Overridden to return either the user-established size
         set by calling "setSize",
       or the size as determined by calling "AdjustSize".
       Must be called after component has had a graphics
       context established for it.

       @return Dimension representing the component's preferred size
     */

    public java.awt.Dimension getPreferredSize()
    {
     if( getSize().width != 0  &&  getSize().height != 0 )
            return getSize();
     else   return adjustSize( getGraphics() );
    }


   /**
     * Overridden to call repaint in case change of enabled
     * state affects the look of the lightweight component.
    */
   boolean state = true;

   public void setEnabled( boolean s )
    {
      if(state != s)
      {
        state = s;
        super.setEnabled(s);
        repaint();
      }
    }

  /** Set the tool tip text */
  public void setTip(String str) { tip = new Tip(this, str); }

 }
