/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;
import java.util.*;


public class DComboBox extends Panel implements ActionListener, MouseListener
{

  /** The command name of the action event fired by this component */
    public String actionCommand;
  /** The action listener to keep track of listeners for our action event */
    public ActionListener actionListener = null;
  /** The text field part of this DComboBox */
    public ComboField editBox;
  /** The button on the right part of this DComboBox that controls dopping the list */
    public ComboButton arrow;
  /** The list part of this DComboBox */
    public ComboList list;
  /** True if the mouse is over the DComboBox*/
    transient public boolean bOverList   = false;
  /** True if the list is currently dropped (displayed) */
    transient public boolean bDown = false;
  /** The latest user-entered text */
    transient public String  sLastText;
  /** If true this DComboBox has a user-editable text field */
    public boolean bEditable;
  /** If true this DComboBox is searchable */
    public boolean bSearchable;
  /** If true search compares are case-sensitive */
    public boolean bCaseSensitive;
  /** The x parameter value in the latest reshape() call */
    public int ix;
  /** The y parameter value in the latest reshape() call */
    public int iy;
  /** The width parameter value in the latest reshape() call */
    public int iwidth;
  /** The current height of the list */
    public int lheight;
  /** The dimension of the arrow button  */
    public int arrowDim;
  /** The distance between the direction button and the edit text box */
    public int hPad;
  /** The distance between the edit text box and the drop down list */
    public int vPad;
  /** If true the horizontal scroll bar is shown as needed */
    public boolean horizScroll = false;
  /** If true the vertical scroll bar is shown as needed */
    public boolean vertiScroll = true;
  /** The case-normalized user-entered string used to match during searches */
    public String sSearchText;
  /** The list item string used to match during searches */
    public String sSearchItem;
  /** The user-entered string used to match during searches Not case normalized */
    public String sSearchOrig;
  /** Where the selected area starts in the user-entered text */
    public int searchIndex;
  /** The number of items in the list to seach (all) */
    public int searchLen;
  /** The list item currently being checked for search match */
    public int searchI;

  /** True if not running under Windows and Mac */
    transient public static boolean bOsFlag;


//    private Mouse	mouse	= null;
    private KeyList	keyList	= null;
    private KeyBox	keyBox	= null;
    //private Focus	focus	= null;
    private Item	item	= null;

    static { bOsFlag = !OS.isWindows() && !OS.isMacintosh();}

      // Rajout
    public int MAXITEMS = 16;
    public int baseSize = 0;


    boolean traversable;
   /** Constructs a new default DComboBox  not editable, not searchable */
    public DComboBox() { this(true, false, 0); }
    public DComboBox(int columns) { this(true, false, columns); }

  // private static Set forwardTraversalKeys, backwardTraversalKeys;

   /** Constructs a new editable and/or searchable DComboBox */
    public DComboBox(boolean editable, boolean searchable, int columns)
    {
    	vPad = 0;
    	hPad = 1;

	super.setLayout(null);

	editBox = new ComboField("", columns);
	editBox.addMouseListener(this);

	arrow = new ComboButton(DirButton.DOWN);
	arrow.setShowFocus(false);
	arrow.setEnabled(false);
	arrow.setBevelHeight(2);
	arrow.setComboEnabled(false);
	arrow.setBorderEnabled(false);
	arrow.addActionListener(this);

	list = new ComboList( true );
	list.setVisible(false);
	list.setComboMode(true);
	list.setShowHorizontalScroll(false);
	list.setShowVerticalScroll(true);
	list.addMouseListener(this);
	add(list);

	add(editBox);
	add(arrow);

	lheight = 0;

        traversable = true;
	setEditable(editable);
	setSearchable(searchable);
	bCaseSensitive = true;

/*
        setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,
                 forwardTraversalKeys);
        setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,
                 backwardTraversalKeys);
*/

    }



    public void setShowFocus(boolean f)
    {
      traversable = f;
      arrow.setShowFocus(f);
    }

    public boolean isFocusable() { return traversable; }


    /**
       DGS
       dessin flche
    */

    public void setArrowStyle(boolean inside, boolean border, int bev)
    {
     arrow.setComboEnabled(inside);
     arrow.setBorderEnabled(border);
     arrow.setBevelHeight(bev);
    }

    /** Conditionally sets the DComboBox to be user-editable  */

    public void setEditable(boolean editable)
    {
        bEditable = editable;
        editBox.setEditable(bEditable || bSearchable);
    }

    /**
     * Conditionally sets the DComboBox to be searchable.
     * A searchable DComboBox allows the user to enter text into
     * the text field as long as it matches one of the existing list items.
     * This allows the user to select from a non-editable list by typing instead
     * of dropping-down the list and selecting with the mouse.
     */

    public void setSearchable(boolean searchable)
    {
        bSearchable = searchable;
        editBox.setEditable(bEditable || bSearchable);
    }


   /** Conditionally set the DComboBox to be case sensitive during searches */

    public void setCaseSensitive(boolean cs) { bCaseSensitive = cs; }

    /**
     * Sets whether or not the horizontal scrollbar should be made visible when
     * necessary or should never be made visible.
    */
    public void setShowHorizontalScroll(boolean cond)
    {
       if(horizScroll != cond)
       {
          horizScroll = cond;
          list.setShowHorizontalScroll(cond);
       }
    }

    /** Gets the current horizontal scrollbar display option */

    public boolean isShowHorizontalScroll() { return horizScroll;}

   /**
     * Sets whether or not the vertical scrollbar should be made visible when
     * necessary or should never be made visible.
    */

    public void setShowVerticalScroll(boolean cond)
    {
    	if(vertiScroll != cond)
    	{
          vertiScroll = cond;
          list.setShowVerticalScroll(cond);
	}
    }

   /** Gets the current vertical scrollbar display option */

    public boolean isShowVerticalScroll() { return vertiScroll; }


   /**
     * Adds the string array to the list.
     * @param items items to add to the list
     */
   public void setListItems(String[] items)
   {
      reset();
      for (int i = 0; i < items.length; ++i)
       {
         addItem(items[i]);
       }
    }

    /**
     * Returns the current list as an array.
     * @see #setListItems
     */
    public String[] getListItems()
    {
        int len = countItems();

        String[] items = new String[len];

        for (int i = 0; i < len; ++i)
            items[i] = getListText(i);

        return items;
    }

    /** Sets the font of the component  */
    public synchronized void setFont(Font newFont)
    {
	super.setFont(newFont);
	editBox.setFont(newFont);
	list.setFont(newFont);
	repaint();
    }

    /**
     * Returns the text associated with a drop-down item.
     * @return the text associated with the drop-down item at index
     * @see #setListText(int, java.lang.String)
     */
    public String getListText(int index) { return list.getText(index); }

    /**
     * Sets the text in the edit field portion of the DComboBox
     */
    public void setText(String newText)
    {
	  editBox.setText(newText);
    }

    /** Returns the text in the edit field portion of the DComboBox */

    public String getText()
    {
        verify();
        return editBox.getText();
    }

    /** Returns the selected item on the drop-down list */
    public String getSelectedItem()  {  return list.getSelectedItem(); }

    /**
     * Gets the zero-relative index of the currently selected item in the drop-down list.
     * @return the zero-relative index of the current selection, or -1 if no item is selected
     * @see #getSelectedItem
     */
    public int getSelectedIndex() { return list.getSelectedIndex(); }

    /**
     * Returns the selection state of the list item at the given zero-relative index.
     */
    public synchronized boolean isSelected(int index)
    {
        return list.isSelected(index);
    }

    /**
     * Determines whether the DComboBox should be the first component added to
     * its container, or the last.
     * @see #addLastBottomUp
     */
    public static boolean addFirstTopDown()
    {
        return !addLastBottomUp();
    }

    /**
     * Determines whether the DComboBox should be the last component added to
     * its container, or the first.
     * @see #addFirstTopDown
     */
    public static boolean addLastBottomUp()
    {
    	return OS.isFirstDrawnOverSecond();
    }

    /**
     * Use this method to set a flag for use in your handleEvent to determine if
     * the DComboBox needs help in "un-dropping".  This will improve useability
     * under platforms which do not provide proper focus events.  For best
     * performance assign a variable in your class once to avoid repeated function
     * calls to this method, like this:
     * <pre>
     *   boolean bNeedsPlatformHelp = DComboBox.needsPlatformHelp();
     *
     *   public boolean handleEvent(Event evt)
     *   {
     *       switch (evt.id)
     *       {
     *           case Event.MOUSE_UP:
     *               // do it on mouse up, which indicates a full click somewhere other than
     *               // the DComboBox that could be currently dropped down.
     *               if (bNeedsPlatformHelp && DComboBox.isListDown)
     *                   DComboBox.setListDown(false);
     *               break; // or other MOUSE_UP processing as necessary
     *
     *           ...
     *       }
     *
     *       ...
     *   }
     * </pre>
     * @see #setListDown
     */

    public static boolean needsPlatformHelp() { return bOsFlag; }

    /** Clears all items from the drop-down list */
    public void clear()
    {
        list.clear();
        baseSize = 0;
    }

    /** Clears all items and the edit box */
    public void reset()
    {
       list.clear();
       editBox.setText("");
    }

    /**
     * Adds the specified item to the end of the drop-down list and enables the item.
     */
    public synchronized void addItem(String item)
    {
        addItem((Image)null, item, true);
    }

    /**
     * Adds the specified item to the end of the drop-down list and conditionally enables it.
     * @param item the item to be added
     * @param bEnabled true = enable the item, false = disable it
     */
    public synchronized void addItem(String item, boolean bEnabled)
    {
        addItem((Image)null, item, bEnabled);
    }

    /**
     * Adds the specified item with an image to the end of the drop-down list and enables it.
     * @param image the image to display on the item line
     * @param item the item to be added
     */
    public synchronized void addItem(Image image, String item)
    {
        addItem(image, item, true);
    }

    /**
     * Adds the specified item with an image to the end of the drop-down list and
     * conditionally enables it.
     * @param image the image to display on the item line
     * @param item the item to be added
     * @param bEnabled true = enable the item, false = disable it
     */
    public synchronized void addItem(Image image, String item, boolean bEnabled)
    {
        list.addItem(image, item, bEnabled);
        arrow.setEnabled(true);
    }



   //    Add a new element not in list


    public synchronized boolean addUnicItem(String item)
    {
      // Dja dans la liste?
      int siz = countItems();
      for(int i = 0; i < siz; i++)
       {
        if(Sort.superEquals(getListText(i), item) == true) return false;
       }

      if(siz >= MAXITEMS)
       {
        delItem(baseSize);
        siz = MAXITEMS - 1;
       }

      // Sinon ajoute
      list.addItem(null, item);
      return true;
    }


    /** Returns the total number of items in the drop-down list */
    public int countItems() { return list.countItems(); }

    /** Changes the image associated with a drop-down item  */
    public void setImage(int index, Image image)
    {
        list.setImage(index, image);
    }

    /** Changes the text associated with a drop-down item  */
    public void setListText(int index, String text)
    {
        list.setText(index, text);
    }

    /**
     * Enables a drop-down list item at a given zero-relative index.
     * @param index the zero-relative index of the drop-down list item
     */
    public void setEnabled(int index)
    {
        setEnabled(index, true);
    }


    /**
     * Conditionally enables a drop-down list item at a given zero-relative index.
     * @param index the zero-relative index of the drop-down list item
     * @param cond true = enable the item, false = disable it
     */
    public void setEnabled(int index, boolean cond)
    {
	list.setEnabled(index, cond);
    }


    /** Deletes an item from the drop-down list */
    public synchronized void delItem(int position)
    {
        list.delItem(position);

        if (list.countItems() == 0)
        {
          arrow.setEnabled(false);
          setListDown(false);
        }
    }

    /**
     * Deletes multiple items from the drop-down list.
     * @param start the starting zero-relative index of the items
     * @param end the ending zero-relative index of the items
     */
    public synchronized void delItems(int start, int end)
    {
        list.delItems(start, end);

        if (list.countItems() == 0)
        {
            arrow.setEnabled(false);
            setListDown(false);
        }
    }

    /** Deletes the currently selected item from the list */
    public synchronized void delSelectedItem()
    {
        list.delSelectedItems();
    }

   /**
     * Selects the drop-down list item with the specified zero-relative index.
     * @param index the zero-relative index of the drop-down list item to select
     * @see #deselect
     */
    public synchronized void select(int index)
    {
        if (index >= 0 && index <= list.countItems())
        {
            list.setSelectedIndex(index, true);
            editBox.setText(list.getSelectedItem());

            if (bOsFlag)  repaint();
        }
    }

    /**
     * Selects the first drop-down item in the list which has exactly matching text.
     */
    public void select(String str)
    {
        list.setSelectedItem(str, true);
        editBox.setText(list.getSelectedItem());

        if (bOsFlag)  repaint();
    }

    /**
     * Deselects the drop-down list item at the specified zero-relative index.
     */
    public synchronized void deselect(int index)
    {
        list.setSelectedIndex(index, false);
        editBox.setText("");
    }

    /**
      Handles dropping and "un-dropping" the DComboBox list.
      @param isDown true to drop the list, false to "un-drop" the list.

    */
    public void setListDown(boolean isDown)
    {
      if (bDown != isDown)
      {
        bDown = isDown;

        if (bDown)
        {
            list.setVisible(true);
            vPad = 1;
        }
        else
        {
            list.setVisible(false);
            vPad = 0;
        }
        reshapeInternals();
        super.setBounds(ix, iy, iwidth, lheight + arrowDim + vPad);
        invalidate();
      }
   }

    /** Is the list up or down?  */

    public boolean isListDown() { return bDown;  }


    /**
     * Moves and/or resizes this component.
     * x      horizontal position in the parent's coordinate space
     * y      vertical position in the parent's coordinate space
     * width  the new width
     * height the new height
     */
    public synchronized void setBounds(int x, int y, int width, int height)
    {
	super.setBounds(x, y, width, height);
        ix = x;
        iy = y;
        iwidth	= width;
        reshapeInternals();
    }

  /**
    * Paints this component using the given graphics context.
    */

    public void update(Graphics G) { paint(G); } // DGS

    public void paint(Graphics g)
    {
	super.paint(g);
        reshapeInternals();
    }

   /**
     * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     * Preferred size is .
     */
    public Dimension getPreferredSize()
    {
    	reshapeInternals();
	int editBoxPreferredWidth = editBox.getPreferredSize().width;

    	if(bDown)
	   return new Dimension(editBoxPreferredWidth + arrowDim + hPad, arrowDim + vPad + lheight);
    	else
    	  return new Dimension(editBoxPreferredWidth + arrowDim + hPad, arrowDim);
    }

    /**
     * Returns the minimum dimensions to properly display this component.
     * Returns the results of a call to getPreferredSize.
      */
    public Dimension getMinimumSize()
    {
    	return getPreferredSize();
    }

    /**
     * Sets the command name of the action event fired by this button.
     * @param command The name of the action event command fired by this button
     * @exception PropertyVetoException
     * if the specified property value is unacceptable
     */
    public void setActionCommand(String command)
    {
        actionCommand = command;
    }

    /** the command name of the action event fired by this button  */
    public String getActionCommand() { return actionCommand; }

	/**
	 * Tells this component that it has been added to a container.
	 * This is a standard Java AWT method which gets called by the AWT when
	 * this component is added to a container. Typically, it is used to
	 * create this component's peer.
	 *
	 * It has been overridden here to hook-up event listeners.
	 *
	 * @see #removeNotify
	 */
   public synchronized void addNotify()
   {
	super.addNotify();

	//Hook up listeners
	if (keyList == null)
	{
		keyList = new KeyList();
		list.addKeyListener(keyList);
	}
	if (keyBox == null)
	{
		keyBox = new KeyBox();
		editBox.addKeyListener(keyBox);
	}
	if (item == null)
	{
		item = new Item();
		list.addItemListener(item);
	}
        /*
	if (focus == null)
	{
		focus = new Focus();
		addFocusListener(focus);
		arrow.addFocusListener(focus);
		editBox.addFocusListener(focus);
	}
       */
  }

 /**
   * Tells this component that it is being removed from a container.
   * It has been overridden here to unhook event listeners.
   */
  public synchronized void removeNotify()
  {
	//Unhook listeners
	if (keyList != null)
	{
		list.removeKeyListener(keyList);
		keyList = null;
	}
	if (keyBox != null)
	{
		editBox.removeKeyListener(keyBox);
		keyBox = null;
	}
	if (item != null)
	{
		list.removeItemListener(item);
		item = null;
	}
        /*
	if (focus != null)
	{
		removeFocusListener(focus);
		arrow.removeFocusListener(focus);
		editBox.removeFocusListener(focus);
		focus = null;
	}
       */

	super.removeNotify();
  }

 /**
   * Takes no action.
   * Since layout managers CANNOT BE USED with this container the standard
   * setLayout has been OVERRIDDEN for this container and does nothing.
   **/
   public void setLayout(LayoutManager lm)  {  }

   public void validate()
    {
	super.validate();
	reshapeInternals();
    }

    /**
     * Adds the specified action listener to receive action events
     * @param l the action listener
     */
     public void addActionListener(ActionListener l)
     {
	actionListener = AWTEventMulticaster.add(actionListener, l);
     }

    /**
     * Removes the specified action listener so it no longer receives
     * action events.
     * @param l the action listener
     */
   public void removeActionListener(ActionListener l)
   {
      actionListener = AWTEventMulticaster.remove(actionListener, l);
   }



  /**
    * Mouse Event handling.
    */
   boolean wasPressed = false;

  /**
    * Handles Mouse Entered events
    * @param e the MouseEvent
     */
   public void mouseEntered(MouseEvent e)
   {
	Object source = e.getSource();
	if(source == list)
		bOverList = true;
   }

  /**
    * Handles Mouse Exited events
    * @param e the MouseEvent
    */
   public void mouseExited(MouseEvent e)
    {
	//We are only listening to the list...
	bOverList = false;
    }

   public void mousePressed(MouseEvent e)
    {
 	Object source = e.getSource();
	wasPressed = (source == editBox);
    }

   public void mouseReleased(MouseEvent e)
   {
	Object source = e.getSource();
	if(source == editBox && wasPressed && !(bEditable || bSearchable))
	{
		setListDown(!bDown);
	}
	wasPressed = false;
   }

  public void mouseClicked(MouseEvent e) { }

	/** This is the Adjustment Event handling inner */


   public void actionPerformed(ActionEvent e)
   {
      Object source = e.getSource();

      if(source instanceof DirButton)
	{
	 setListDown(!bDown);
	}
      else if(e.getActionCommand().equals("ImageSelected"))
	{
	  sourceActionEvent("ImageSelected");
	}
   }

    /** For handling KeyEvents from the editBox */

    public class KeyBox extends java.awt.event.KeyAdapter implements java.io.Serializable
    {
       public void keyPressed(KeyEvent e)
       {
          int key =  e.getKeyCode();

          if(e.isControlDown())
          switch(key)
          {
              case KeyEvent.VK_A:
                   addUnicItem(editBox.getText());
                   break;
              case KeyEvent.VK_Z:
                   clear();
                   break;
              case KeyEvent.VK_Y: //deleteInList();
                   break;
              case KeyEvent.VK_X: editBox.setText("");
                   break;
           }
          switch(key)
           {
              case KeyEvent.VK_ENTER:
                   //addUnicItem(editBox.getText());
                   break;
              default:
                   sLastText = editBox.getText();
                   break;
           }
 	}
    }


	/**
	 * This is the Key Event handling innerclass for handling
	 * KeyEvents from the list.
	 */

  public class KeyList extends java.awt.event.KeyAdapter implements java.io.Serializable
  {

	/**
	 * Handles Key Released events
	 * @param e the KeyEvent
	 */
	public void keyReleased(KeyEvent e)
	{
           //We are only listening to the list...
           switch(e.getKeyCode())
           {
               case KeyEvent.VK_HOME:
                    list.setSelectedIndex(0, true);
                    break;
               case KeyEvent.VK_END:
                    list.setSelectedIndex(list.countItems() - 1, true);
                    break;
               case KeyEvent.VK_PAGE_UP:
               case KeyEvent.VK_UP:
                    list.setSelectedIndex(list.getSelectedIndex() - 1, true);
                    break;
               case KeyEvent.VK_PAGE_DOWN:
               case KeyEvent.VK_DOWN:
                    list.setSelectedIndex(list.getSelectedIndex() + 1, true);
                    break;
             }

            editBox.setText(list.getSelectedItem());
            editBox.selectAll();
            verify();
	}
   }

   /** Focus Event handling */

/*
    public class Focus implements java.awt.event.FocusListener, java.io.Serializable
     {
       public void focusGained(FocusEvent e)
 	{
           Object source = e.getSource();
           if (source instanceof DComboBox && !bDown)
            {
                editBox.selectAll();
                editBox.requestFocus();
            }

            if (bOsFlag)
              if (source == editBox)
                setListDown(false);
	}

    public void focusLost(FocusEvent e)
     {
            if ((e.getSource() == arrow) && !bOverList)
            {
               setListDown(false);
            }
    }
  }
*/

   /** This is the Item Event handling innerclass */

   public class Item implements java.awt.event.ItemListener, java.io.Serializable
   {
	/**
	 * Handles Item State Changed events
	 * @param e the ItemEvent
	 */
       public void itemStateChanged(ItemEvent e)
	{
           editBox.setText(list.getSelectedItem());
	   sourceActionEvent();
           setListDown(false);
	}
   }

    /**
     * Fire an action event to the listeners
     * @param the string containing the command to send
     * with the event.
     * @see #sourceActionEvent()
     */
   public void sourceActionEvent(String command)
   {
	if (actionListener != null)
		actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
   }

    /**
     * Fire an action event to the listeners
     * with the actionCommand
     * @see #sourceActionEvent(java.lang.String)
     */
        public void sourceActionEvent()
	{
		sourceActionEvent(actionCommand);
	}

  /**
    * A utility routine to resize the sub-components that make up a DComboBox.
    */
    public void reshapeInternals()
    {
	Dimension	s						= getSize();
	Dimension	arrowSize				= arrow.getPreferredSize();
	Dimension	editBoxPreferredSize	= editBox.getPreferredSize();

	arrowDim = Math.max(arrowSize.width, arrowSize.height);

	if (editBoxPreferredSize.height > arrowDim)
		arrowDim = editBoxPreferredSize.height;


	if (OS.isMacintosh())
	{
		arrowDim++;
		editBox.setBounds(0,1, s.width  + 1 - arrowDim - hPad, editBoxPreferredSize.height);
	}
	else
		editBox.setBounds(0,0, s.width + 1 - arrowDim - hPad, editBoxPreferredSize.height);

        //DGS dcal  gauche
	arrow.setBounds(s.width - arrowDim , 0, arrowDim, arrowDim);

	if (bDown)
	{
  	  if (getParent() != null)
          {
            lheight = Math.min(getParent().getBounds().height - iy - arrowDim - vPad, list.getMinimumSize().height);
          }
	  else
	  {
	    lheight = list.getMinimumSize().height;
	  }

	  list.setBounds(0, arrowDim + vPad, s.width, lheight);
	 }
	 else
	   lheight = 0;
	}

    /**
     * Verify that at least one list item starts with the edit field's text.
     * If a match is found, the edit field's text is set to that item's value.
     * This is done only if searching is enabled.
     */
    public void verify()
    {
        if (bSearchable == true)
        {
            searchIndex = editBox.getSelectionStart();
            sSearchOrig = editBox.getText().substring(0, searchIndex);

            if (bCaseSensitive)
            {
                sSearchText = sSearchOrig;
            }
            else
            {
                sSearchText = sSearchOrig.toUpperCase();
            }

            searchLen = list.countItems();

            if (sSearchText.length() > 0)
            {
                for(searchI = 0; searchI < searchLen; ++searchI)
                {
                    if (bCaseSensitive)
                    {
                        sSearchItem = list.getText(searchI);
                    }
                    else
                    {
                        sSearchItem = list.getText(searchI).toUpperCase();
                    }

                    if (sSearchItem.startsWith(sSearchText))
                    {
                        editBox.setText(sSearchOrig);
                        list.setSelectedIndex(searchI, true);
                        editBox.select(searchIndex, searchIndex);
                        break;
                    }
                }

                if (!bEditable && (searchI == searchLen)) // !editable && search failed
                {
                    editBox.setText(sLastText);
                    editBox.select(searchIndex-1, searchIndex-1);
                }
            }
        }

        sLastText = editBox.getText();
    }

    /**
      * Dfinit le nombre max d'lments dans la liste
      */

    public int setMaxItems(int val)
    {
      int i = MAXITEMS;
      MAXITEMS = val;
      return i;
    }

    /**
       DGS
      */
    public void setBaseSize(int n)
    {
      if(n >= MAXITEMS) n = MAXITEMS;
      baseSize = n;
    }

    public int getMaxItems()  { return MAXITEMS; }

    public int getBaseSize() { return baseSize; }


    public void setBackground(Color c)
    {
     super.setBackground(c);

     arrow.setBackground(c);
     arrow.setComboColors(c);

     editBox.setBackground(c);
     list.setBackground(c);

    }

    public void setForeground(Color c)
    {
     super.setForeground(c);
     arrow.setForeground(c);
    }


    // override
    public void setFieldsBackground(Color c)
    {
      editBox.setBackground(c);
      list.setBackground(c);
    }

    public void setFieldsForeground(Color c)
    {
      editBox.setForeground(c);
      list.setForeground(c);
    }


    public void setButtonBackground(Color c)
    {
      arrow.setBackground(c);
    }

    /**
       SetStyle concerne le contenu
    */
    public void setStyle(Style s)
    {
      setFieldsBackground(s.background);
      setFieldsForeground(s.foreground);
      setFont(s.font);
    }

    /**
       DGS
       Efface un lment par la boite d'dition
      */

    public void deleteInList()
    {
      String str = editBox.getText();

      list.setSelectedItem(str, true);
      int i = list.getSelectedIndex();
      delSelectedItem();

      if(i >= list.countItems()) i--;
      if(i < 0) str = "";
      else  str = list.getText(i);

      editBox.setText(str);
    }


   /** Change la taille de liste */
   public void Resize(int newsize)
   {
     int oldsize = getMaxItems();
     int newbase;

     if(OS.isWindows()) newbase = baseSize;
                   else newbase = Preferences.MemBaseSize;

     int oldbase = getBaseSize();

     if((newsize == oldsize) && (newbase == oldbase)) return;

     // On prend la liste pour la trier en cas de changement de base
     String [] clist = getListItems();
     int l = clist.length;

     newbase = SortDir.separeDrives(clist, 0, l); // Spare et trie les drives
     //SortDir.sortList(clist, 0, newbase);

     clear();
     // On ne reprend que la partie utile
     if(newsize < l) l = newsize;

     for(int i = 0; i < l; i++)
      addItem(clist[i]);

     setMaxItems(newsize);
     setBaseSize(newbase);

   }

   /** Mets les units en tte */
   void separeDrives()
   {
     String [] clist = getListItems();
     int l = clist.length;

     int base = SortDir.separeDrives(clist, 0, l);
     // Reconstitue la liste affiche
     clear();
     for(int i = 0; i < l; i++)
       addItem(clist[i]);

     setBaseSize(base);
   }



  void setBaseSize()
  {
   setBaseSize(SortDir.countDrives(getListItems()));
  }

    /** Cherche un lment */

    public int search(String str)
    {

      if (str.length() < 1) return -1;

      int L = list.countItems();

      for(int i = 0; i < L; i++)
       {
         if (bCaseSensitive)
          {
             if (list.getText(i).equalsIgnoreCase(str)) return i;
          }
         else
          {
             if (list.getText(i).equals(str)) return i;
          }
       } // for

      return -1;
    }


// EMBEDDED CLASSES (for focus)

public class ComboField extends TextField
  {
    public ComboField(String s) { super(s); }
    public ComboField(String s, int nc) { super(s, nc); }
    public boolean isFocusable()
    {
      return traversable;
    }
  }

public class ComboButton extends DirButton
  {
    public ComboButton(int i) { super(i); }
    public boolean isFocusable()
    {
      return traversable;
    }
  }

public class ComboList extends DListBox
  {
    public ComboList(boolean b) { super(b); }
    public boolean isFocusable()
    {
      return traversable;
    }
  }

}

