/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.io.File;



public class DirBox extends DListBox
{
  VectorBoolean VType;
  VectorLong VSize;
  VectorLong VDate;

  public long TotalSize;
  public long TaggedSize;

  int namePos = 20;
  int sizePos;    // position size
  int datePos;    // position date
  int endPos;     // position fin de ligne
  int charWidth = 8;
  int dateWidth;  // largeur selon fonte
  int sizeWidth;

  final static int sizeLen = 9;
  final static int dateLen = 11;
  //static SimpleDateFormat df = new SimpleDateFormat("dd MMM yyyy");
  static DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, 0);
  int oldWidth = 0;     // Largeur d'cran
  boolean restarted;    // Font change


  /** Constructeurs  */



  DirBox(boolean mselect)
   {
     super(mselect);
     restarted = true;
   }

  DirBox()  { this(false); }


    /** Barrer l'item */

 public void crossOut(int index)
  {
   if (validIndex(index))
     {
         ListItem tempItem = ((ListItem)items.elementAt(index));
         tempItem.deleted = true;
         tempItem.bDirty = true;
     }
  }

 public void crossOut(int index, boolean val)
  {
    if (validIndex(index))
     {
      ListItem tempItem = ((ListItem)items.elementAt(index));
      tempItem.deleted = val;
      tempItem.bDirty = true;
     }
  }


 /** Fichier barr? */
 public boolean isCrossOut(int index)
  {
   if (validIndex(index))
     {
       ListItem tempItem = ((ListItem)items.elementAt(index));
       return(tempItem.deleted);
     }
   return false;
 }


/** S'il existe au moins un fichier barr  */

public boolean isCrossOut()
 {
  int l = items.size();

  for(int index = 0; index < l; index++)
   {
     ListItem tempItem = ((ListItem)items.elementAt(index));
     if(tempItem.deleted) return true;
   }
  return false;
}


/**
  * Selects all the items in the list.
  */
public synchronized void selectAllFiles(int MAXINDIRS)
  {
    if (!bMultipleSelections)  return;

    int s = items.size();
    ListItem li;
    for (int i = MAXINDIRS; i < s; i++)
     {
      li = (ListItem)items.elementAt(i);
      if (li.bEnabled) li.bSelected = true;
     }

    countSelected = s;
    lastSelected = s - 1;
    repaint();
 }



 public void setFont(Font f)
 {
   super.setFont(f);
   restarted = true;
 }


 public void adjustFontWidth()
  {
    FontMetrics fm = getFontMetrics(getFont());
    charWidth = fm.charWidth('0');
    sizeWidth = sizeLen * charWidth;
    dateWidth = fm.stringWidth("00/00/2010 00:00");
    restarted = false;
  }


 /**
     Redefine field of names according to fullLineLength
     and size of the window
 */

 int fullLineLength;

 public void adjustWidth()
 {
    final int ENDMARGIN = 28;
    final int SIZEMARGIN = 12;
    final int BORDERS = 4;
    Dimension size = getSize(); // Taille de la fentre

    if(restarted) adjustFontWidth();

    fullLineLength = longestLineValue;

    int width      = size.width - barSize - BORDERS;
    int rightWidth = sizeWidth + dateWidth + SIZEMARGIN + ENDMARGIN;
    int leftWidth  = width - rightWidth;

    int minPart    = 20 * charWidth;
    if(leftWidth < minPart) leftWidth = minPart;
    if((fullLineLength + LINE_SLOP) < leftWidth) fullLineLength = leftWidth;

    sizePos = fullLineLength;                    // pos of size
    datePos = sizePos + sizeWidth + SIZEMARGIN;  // pos date
    endPos  = datePos + dateWidth + ENDMARGIN;   // extra width for size/date

    fullLineLength = endPos - 4;
 }


    /**
       return the list of files
    */
    public String[] getFiles(String dir)
    {
      int len = countItems();
      VectorString items = new VectorString(len);

     for (int i = 0; i < len; ++i)
     {
       String fic = getText(i);
       if (!(new File(dir, fic).isDirectory()))
           items.addElement(getText(i));
     }
     return items.toArray();
    }


    public void update(Graphics g)
    {
      super.update(g);
      //if (!bBlockPaint && !bInternalBlockPaint)  paint(g);
    }

    public synchronized void paint(Graphics g)
     {
        ListItem item;

        int nItems;
        int xSelectStartLoc;
        int i, j;
        int rows;
        int vWid = 0;
        int hHt = 0;
        int gridAdj = 0;
        int yLoc = 0;
        int ySLoc = 0;
        boolean bShowV = false;
        boolean bShowH = false;


        // DGS recadrement donnes extra
        //Dimension size = getSize();
        if(listChanged) adjustWidth(); // dpend de fullLineLength

	//Make sure cached colors are correct.
	Color curBackground = getBackground();
	if (!Tool.objectsEqual(curBackground , cachedBackground))
	{
		cachedBackground = curBackground;
		calculateBorderColors(PanelColor);
	}

        // Set size of ListBox
        Rectangle rect = getBounds();
        font = g.getFont();
        fm = g.getFontMetrics(font);
        fontHeight = fm.getHeight();
        fontDescent = fm.getDescent();

        nItems = items.size();

        if (bCellBorders) cellHt = fontHeight+5;
        else              cellHt = fontHeight+1;

        if (nItems == 0)
        {
            nTopRow = 0;
            visibleRows = 0;
            bShowH = false;
            bShowV = false;
            xCoord = 0;
        }
        else
        {
          if (bAllowShowHBar && (fullLineLength > (rect.width - borderWidth)) )
           {
             bShowH = true;
             hHt = barSize;
           }
          else
           {
             bShowH = false;
             hHt = 0;
           }

         rows = (rect.height - hHt - borderWidth - yAdj) / cellHt;

         if (bAllowShowVBar && ( nItems > rows ))
           {
             bShowV = true;
             vWid = barSize;
             // if no horiz. bar perhaps vert. bar make it necessary
             if (!bShowH)
                {
                  if( bAllowShowHBar && (fullLineLength > (rect.width - borderWidth - vWid)) )
                    {
                        bShowH = true;
                        hHt = barSize;
                        rows = (rect.height - hHt - borderWidth - yAdj) / cellHt;
                    }
                }
            }
            else
            {
                bShowV = false;
                vWid = 0;
            }

            if (visibleRows != rows)
            {
                visibleRows = rows;
                bAllDirty = true;
            }

            if (bShowV)
            {
               VBar.setBounds(rect.width - barSize - halfBorderWidth,
                               halfBorderWidth,
                               barSize,
                               rect.height - borderWidth - hHt
                               );

                VBar.setValues(nTopRow, visibleRows, 0, nItems);

                VBar.setBlockIncrement(visibleRows);
                lWidth = rect.width - vWid - borderWidth;

                if (!bVBarVisible)
                {
                    bVBarVisible = true;
                    VBar.setVisible(true);
                }
            }
            else
            {
                lWidth = rect.width - borderWidth;
                if (bVBarVisible)
                {
                    bVBarVisible = false;
                    VBar.setVisible(false);
                }
            }

            if (bShowH)
            {
                HBar.setBounds(halfBorderWidth, rect.height-barSize-halfBorderWidth, rect.width-borderWidth-vWid, barSize);
                HBar.setValues(-xCoord, lWidth, 0, fullLineLength);
                HBar.setBlockIncrement(lWidth);

                if (!bHBarVisible)
                {
                    bHBarVisible = true;
                    HBar.setVisible(true);
                }
            }
            else
            {
                if (bHBarVisible)
                {
                    bHBarVisible = false;
                    HBar.setVisible(false);
                }
            }
        }
/*
        if (nItems == 0 || bAllDirty)
        {
           g.clearRect(halfBorderWidth, halfBorderWidth, rect.width - borderWidth - 1, rect.height - borderWidth - 1);
	}
*/
        if (borderType == BORDER_REGULAR)
        {
            g.setColor(borderDarkerColor);
            g.drawLine(0, 0, rect.width - 1, 0);
            g.drawLine(0, 0, 0, rect.height - 1);

            g.setColor(borderDarkColor);
            g.drawLine(1, 1, rect.width - 2, 1);
            g.drawLine(1, 1, 1, rect.height - 2);

            g.setColor(borderLightColor);
            g.drawLine(1, rect.height - 2, rect.width - 2, rect.height - 2 );
            g.drawLine(rect.width - 2, 1, rect.width - 2, rect.height - 2 );

            g.setColor(borderLighterColor);
            g.drawLine(0, rect.height - 1, rect.width - 1, rect.height - 1 );
            g.drawLine(rect.width - 1, 0, rect.width - 1, rect.height - 1 );
        }

        g.clipRect(halfBorderWidth, halfBorderWidth, lWidth, rect.height - borderWidth);

        if (nTopRow >= nItems)  nTopRow -= visibleRows + 1;

        i = nTopRow;
        j = Math.min(nTopRow + visibleRows, nItems);

        for (int dp = 0; i < j; ++i, ++dp)
        {
            yLoc = ( (dp) * (cellHt) ) + yAdj + halfBorderWidth;
            ySLoc = ( (dp + 1) * (cellHt) ) + yAdj + halfBorderWidth;

            item = (ListItem)items.elementAt(i);
            if (item.bDirty || bAllDirty)
            {
                item.bDirty = false;
                g.clearRect(halfBorderWidth, yLoc, lWidth, cellHt);
            }

            if (item.image != null)
                xSelectStartLoc = halfBorderWidth + IMAGE_WIDTH + 2;
            else
                xSelectStartLoc = halfBorderWidth;

            if (item.bCellBorder) gridAdj = 2;
            else                  gridAdj = 0;

            int tWidth = lWidth  - xCoord - IMAGE_WIDTH - 2;
            if (item.bSelected)
            {
               g.setColor(textHighlight);
               g.fillRect(xCoord + xSelectStartLoc + gridAdj, yLoc, tWidth, cellHt);
               g.setColor(textHighlightText);
            }
            else
            {
               g.setColor(enabledColor);
            }

            int x = xCoord + xSelectStartLoc + gridAdj + 2;
            int y = ySLoc - fontDescent;

            g.drawString(item.sText, x, y);   // Nom du fichier

            if(VType.elementData[i])
            {
             try
             {
              String s = Tool.longToString(VSize.elementAt(i), sizeLen);
              String d = df.format(new Date(VDate.elementAt(i)));

              int depl = sizeWidth - s.length() * charWidth;

              g.drawString(s, x + sizePos + depl, y);
              g.drawString(d, x + datePos, y);
             }
             catch(ArrayIndexOutOfBoundsException aie) { aie.printStackTrace(); }
            }

            if (item.image != null)
            {
               g.drawImage(item.image, xCoord + halfBorderWidth + gridAdj + 2, yLoc+2, IMAGE_WIDTH, cellHt-3, this);
            }

            // trace un trait rouge dessus (si effac)
            if(item.deleted)
            {
             g.setColor(Color.red);
             int yy = yLoc + cellHt / 2;
             g.drawLine( xCoord + halfBorderWidth + gridAdj + 2, yy, lWidth, yy);
            }

            if(item.tagged)
            {
             g.setColor(textHighlight);
             g.drawRect(xCoord + xSelectStartLoc + gridAdj, yLoc + 1, tWidth - xCoord - 2, cellHt - 2);
            }

        }

        bAllDirty = false;
        listChanged = false;
    }


  /**
     Taille cumule des fichiers slectionns
    */
  public long getTaggedSize()
  {
   TaggedSize = 0;

   for(int i = 0; i < countItems(); i++)
    {
     if(isSelected(i))
      TaggedSize += VSize.elementAt(i);
    }
   return TaggedSize;
  }



  /**
     Clear the list and make empty vectors
    */
  public void clear(int capacity)
  {
   super.clear(capacity);
   VType = new VectorBoolean(capacity, 16);
   VSize = new VectorLong(capacity, 16);
   VDate = new VectorLong(capacity, 16);
   TotalSize = 0;
  }


  /**
    Ajout
    */
  public synchronized void addItemDir(int no)
   {
     addItem(no);
     VType.addElement(false);
     VSize.addElement(0);
     VDate.addElement(0);
   }


  public void addItemFile(Image img, int no, long size, long time)
  {
    addItem(img, no);
    VType.addElement(true);
    VSize.addElement(size);
    VDate.addElement(time);
    TotalSize += size;
  }

  /** Suppression */

  public void delItem(int index)
  {
    super.delItem(index);
    TotalSize -= VSize.elementAt(index);

    VType.removeElementAt(index);
    VSize.removeElementAt(index);
    VDate.removeElementAt(index);
  }

  /** Retourne les tailles des items marqus */
  public long[] getSelectedSizes()
   {
    long l[] = new long[countSelected];
    ListItem tempItem;

    if (countSelected > 0)
    {
     int x = 0;
     int s = items.size();

     for (int i = 0; i < s; i++)
      {
       tempItem = (ListItem)items.elementAt(i);
       if (tempItem.bSelected || tempItem.tagged)
        {
         l[x] = VSize.elementAt(i);
         if (++x == countSelected)  break;
        }
      }
    }

   return l;
  }

}
