/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

/**
   Classe Directory
   holds and manages the list of file
   Works with a
   - ListBox for a simple list
   - DirBox for a list with sizes and dates
*/

import java.io.*;
import java.util.*;
import java.awt.*;
import java.net.URL;
import java.util.zip.*;
import java.lang.*;
import java.lang.reflect.Array;

import common.*;

public class Directory
{
  public final int MAXINDISKS = 0;
  public int MAXINDIRS;
  public int MAXINFILES;
  public int MAXINLIST;

  final short TYPE_DISK = 0;
  final short TYPE_FLOPPY = 1;
  final short TYPE_CDROM = 2;
  final short TYPE_REM = 3;
  final short TYPE_MEDIA = 8;
  final short TYPE_DIR = 16;
  final short TYPE_FILE = 24;
  final short TYPE_HTML = 25;
  final short TYPE_EXE = 26;
  final short TYPE_DOC = 27;
  final short TYPE_SOUND = 32;
  final short TYPE_IMAGE = 48;
  final short TYPE_ZIP = 64;

  DirBox  dirbox;
  public Image image;

  Sort sort = null;
  String Contenu[];
  int Idx[];                 // Index to sort names
  VectorLong EntrySizes;
  VectorLong EntryTimes;

  int Index;                 // Position

  String path;               // Directory + filter                                            ,
  String directory;          // Current directory
  String filter;
  boolean updated;           // Dir after changing name


  public String arcname = null;

  // Flags
  public boolean EXTENDED = false;
  public boolean maskOther = false;    // n'affiche que les types reconnus
  public boolean maskImage = false;    // n'affiche que les types reconnus
  public static boolean Sorting = true; // Tri (toujours vrai)

  public boolean zipMode = false;
  public boolean isArchive = false;  // forcer mode archive
  public boolean isImage = false;  // appartient aux images
  public boolean isOther = false;   // appartient aux autres fichiers
  public boolean jarMask = false;    // afficher les jar

  public static String root;

  static DirIcons di = new DirIcons();

  public Directory(boolean mode)
  {
    EXTENDED = mode;
    if(OS.isWindows())
         root = "\\";
    else root = "/";
  };

  public Directory()  { this(false); }

  public int getMax() { return MAXINLIST; }
  public int getNumber() { return MAXINLIST; }
  public String getDir() { return directory; }
  public String getDrive()  { return Path.splitDrive(directory); }
  public String getPath() { return path; }
  public String getFilter() { return filter; }
  public boolean hasFilter()
  {
   if(filter != null)
    if(filter.length() > 0)
     return true;
   return false;
  }
//  public String getName() { return Path.splitFile(path); }

  // La liste sera celle d'une archive
  void setArchive(boolean isArc) { isArchive = isArc;  }

  // Indique qu'on affiche une archive
  void setZipMode(boolean mode) { zipMode = mode; }

  void setFileMask(boolean m)  { maskOther = m;  updated = false; }  // afficher tout sauf fichier
  void setImageMask(boolean m) { maskImage = m; updated = false; } // afficher image seules

  private boolean arcSort = true;
  void setArcSort(boolean b)
  {
   if (b != arcSort)
   {
    arcSort = b;
    updated = false;
   }

  }

  public void setPath(String newpath)
  {
    try
    {
      path = newpath;
      directory = Path.splitPath(newpath);
      filter = Path.splitFile(newpath);
      arcname = filter;
      updated = false;
    }
    catch(NullPointerException e) { System.out.println("Error directory empty"); }
  }


  /** Affecter le nom de l'archive ventuel */
  public void setArcFile(String fic)
  {
    try { arcname = fic; }
    catch(NullPointerException e) {e.printStackTrace(); arcname = ""; }
  }

  /** Crer un sous-rpertoire */
  static public boolean createDir(String path, String dname)
  {
    File f = new File(path, dname);
    if (f.exists())
     return f.isDirectory();

    return f.mkdirs();
  }

  /** Crer un sous-rpertoire */
  static public boolean createDir(String path)
  {
    return (new File(path)).mkdirs();
  }


 /** Changer de lecteur
     Mettre  jour la liste */

  public void goDrive(String newdrive)
  {
    if(directory.startsWith(newdrive) == false) setPath(newdrive);
    setArchive(false);
  }


  public String nullDir()
  {
    if (OS.isWindows()) return "";
    else return "/";
  }

  /**
     Revenir au rpertoire parent
     Retourne le rpertoire courant
   */


  public String goParent()
  {
    String newpath;
    String current = "";

    if(filter.length() > 0)  // masque ou zip, source seulement
    {
     if(zipMode) current = filter; // pour la slection
     setPath(directory);   // dir seul sans filtre
    }
    else // Source ou Target
    {
      File dir = new File(directory);
      newpath = dir.getParent();
      if(newpath == null) return nullDir();
      if(newpath.length() == 0) return nullDir();
      // parent existe
      current = Path.splitLast(directory);

      int i = newpath.lastIndexOf(dir.separator);
      if(i == (newpath.length() - 1))
       newpath = newpath.substring(0, i); // enleve terminateur
      setPath(newpath);
    }

   setArchive(false);
   return current;
  }


  public void goHome()
  {
    if(OS.isWindows())
    {
     String drive = Path.splitDrive(directory);
     if(Path.isValidDrive(drive)) setPath(drive);
     else  setPath(File.separator);
    }
    else
    {
      setPath("/");
    }
    setArchive(false);
  }


  /**  Rpertoire ou fichier vide  */

   static public boolean IsEmpty(File fichier)
   {
     if(fichier.exists())
     {
      if(fichier.isDirectory())
      {
       String test[] = fichier.list();
       if(test != null)
        if(test.length > 0) return false;
      }
      else
      if(fichier.length() > 0) return false;
     }

    return true;
   }


   /** IsZip */

   static public boolean isArchive(String path)
   {
    if (isZip(path))  return true;
    if (isGZip(path)) return true;
    return false;
   }


   static public boolean isZip(String path)
   {
    String ul = path.toLowerCase();

    if(ul.indexOf("*") != -1) return false;  // c'est un masque
    if(ul.endsWith(".zip")) return true;
    if(ul.endsWith(".jar")) return true;
    if(ul.endsWith(".sxw")) return true;
//    if(ul.endsWith(".aaz")) return true;
    return false;
   }


   static public boolean isGZip(String path)
   {
    String ul = path.toLowerCase();
    if(ul.indexOf("*") != -1) return false;  // c'est un masque
    if(ul.endsWith(".gz")) return true;
    if(ul.endsWith(".tgz")) return true;
    return false;
   }


   /** Test si le chemin courant contient un nom zip */
   public boolean isZip()
   {
    if(isArchive) return true;
    if(arcname == null) return false;

    return isZip(arcname);
   }


  /**
     Sort
    */

  private void swapList( int i, int j)
   {
     int T = Idx[i];  Idx[i] = Idx[j];  Idx[j] = T;
   }


 /** Mets les dirs avant */

 public void separeDirs()
  {
    MAXINDIRS = MAXINDISKS;
    int ptr = MAXINDIRS + 1;

    while(ptr <= MAXINLIST)
    {
     if( (new File(directory, Contenu[Idx[MAXINDIRS]])).isDirectory())
     {
      MAXINDIRS++;
      if(ptr == MAXINDIRS) ptr++;
     }
     else
     {
      if(ptr < MAXINLIST)
      if((new File(directory, Contenu[Idx[ptr]])).isDirectory())
       {
        swapList(MAXINDIRS, ptr);
        MAXINDIRS++;
       }
      ptr++;
     }
    } // fin while

    MAXINFILES = MAXINLIST - MAXINDIRS;
  }



  /**
      SortList - Classe rpertoires
   */

  public void sortList() throws Exception
  {
    sort = new Sort(Contenu, Idx);

    if(MAXINDIRS > 0)  SortRange(0, MAXINDIRS);
	if(MAXINFILES> 0)  SortRange(MAXINDIRS, MAXINLIST);
  }


  public void SortRange(int st, int limit)
  {
    sort.insertion(st, limit);
//     sort.quick(st, limit);
  }




  /**
    GetList
    Met le rpertoire dans la variable Contenu[]
  */

  public boolean getList()
  {

    Contenu = null;
    System.gc();

    MAXINDIRS = MAXINDISKS;
    MAXINLIST = 0;

    if(directory == null) directory = Path.getSysPath();  // par dfaut, systeme

    //System.out.println("Directory getList() - " + directory);

    if(EXTENDED)
    {
     EntrySizes = new VectorLong(1024, 16);
     EntryTimes = new VectorLong(1024, 16);
    }

    //System.out.println("d path " + directory);

    if(isZip())
    {
     //System.out.println("Entering zip " + Path.merge(directory, arcname));

     zipMode = true;
     isArchive = false; // ne sert plus
     //int i = 0;

     try
     {
      ZipFile archive = new ZipFile(Path.merge(directory, arcname));
      Enumeration e = archive.entries() ;
      VectorString temp = new VectorString(1024, 16);

      //int j = 0;
      boolean again = true;
      while(e.hasMoreElements() && again)
      {
       try
       {
         String s = e.nextElement().toString();
         //System.out.println("string " + j + ") / " + temp.capacity()); j++;
         temp.addElement(s);
         if(EXTENDED)
          {
           ZipEntry ze = archive.getEntry(s);
           EntrySizes.addElement(ze.getSize());
           EntryTimes.addElement(ze.getTime());
          }
       }
       catch(NoSuchElementException ex)
       {
        System.out.println("No such element");
        again = false;
       }
      }

     Contenu = temp.toArray();
     archive.close();
    } // try
    catch(java.io.IOException e)
     {
       System.out.println("Error in Zip");
       zipMode = false;
     }
    if(!zipMode) return false;

   }  // if is zip
   else // Not a zip archive
    {
     //System.out.println("not in zip");
     zipMode = false;
     arcname = "";
     File path = new File(directory, root);             // crer rpertoire

     if(!OS.isWindows()) System.out.println(directory);
     //System.out.println(path.getPath());
     //System.out.println(path.getParent());
     try
     {
        Contenu = path.list();

        if(EXTENDED)
        {
         for(int ne = 0; ne < Contenu.length; ne++)
         {
           File f = new File(directory, Contenu[ne]);
           EntrySizes.addElement(f.length());
           EntryTimes.addElement(f.lastModified());
         }
        }
     }          // prendre contenu
     catch (SecurityException e)    { e.printStackTrace();  }
     catch (NullPointerException e) { System.out.println("Dir list not filled"); }
     catch (ArrayIndexOutOfBoundsException e) { e.printStackTrace();  }
    }

    if(Contenu == null) return false;

    MAXINLIST = Contenu.length;

    Idx = new int[MAXINLIST];
    for(int k = MAXINDISKS; k < MAXINLIST; k++)  Idx[k] = k;

    //System.out.println("Separe");

    separeDirs();  // Met les dirs avant et dcompte

    if(zipMode)
    {
      if(!arcSort) return true;
    }
    else
    {
     if(!Sorting) return true;
    }

    try { sortList(); }
    catch(Exception e) { System.out.println("Enable to sort"); }

    //System.out.println("get list passed");
    return true;
  }


// Le type INDIRECT

  public int getType(int no)
   {

     if(no < MAXINDISKS) return TYPE_DISK;
     if(no < MAXINDIRS)  return TYPE_DIR;

     no = Idx[no];
     int l = Contenu[no].length();
     int li = Contenu[no].lastIndexOf('.');


     if(l > 5)
     {
      if(li == (l - 5))
      {
       String suffix = Contenu[no].substring(l-4, l);
       if(suffix.equalsIgnoreCase("html")) return TYPE_DOC;
       if(suffix.equalsIgnoreCase("php4")) return TYPE_DOC;
       if(suffix.equalsIgnoreCase("php5")) return TYPE_DOC;
       return TYPE_FILE;
      }
     }

     if(l > 4)
      if(li == (l - 4))
      {
      String suffix = Contenu[no].substring(l-3, l);
      if(suffix.equalsIgnoreCase("zip")) return TYPE_ZIP;
      if(suffix.equalsIgnoreCase("gif")) return TYPE_IMAGE;
      if(suffix.equalsIgnoreCase("jpg")) return TYPE_IMAGE;
      if(suffix.equalsIgnoreCase("pcx")) return TYPE_IMAGE;
      if(suffix.equalsIgnoreCase("png")) return TYPE_IMAGE;
      if(suffix.equalsIgnoreCase("wav")) return TYPE_SOUND;
      if(suffix.equalsIgnoreCase("exe")) return TYPE_EXE;
      if(suffix.equalsIgnoreCase("doc")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("rtf")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("hlp")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("htm")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("php")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("asp")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("wri")) return TYPE_DOC;
      if(suffix.equalsIgnoreCase("mid")) return TYPE_SOUND;

      if(jarMask)
      {
       if(suffix.equalsIgnoreCase("jar")) return TYPE_ZIP;
       if(suffix.equalsIgnoreCase("sxw")) return TYPE_ZIP;
       if(suffix.equalsIgnoreCase("ico")) return TYPE_IMAGE;
       if(suffix.equalsIgnoreCase("raw")) return TYPE_IMAGE;
       if(suffix.equalsIgnoreCase("pal")) return TYPE_IMAGE;
       if(suffix.equalsIgnoreCase("aaz")) return TYPE_ZIP;
      }

      return TYPE_FILE;
     }


     if(l > 3)
      if(li == (l - 3))
     {
      String suffix = Contenu[no].substring(l - 2, l);
      if(suffix.equalsIgnoreCase("au")) return TYPE_SOUND;
//    if(suffix.equalsIgnoreCase("gz")) return TYPE_ZIP;

     }

     return TYPE_FILE;
   }

// Retourne le nom d'un fichier INDIRECT dans la liste

  public String getFile(int i)
  {
    if(i < MAXINLIST) return(Contenu[Idx[i]]);
    else return(null);
  }

  void setCategory(int i)
  {
    isImage = false;
    isOther = false;

    switch(getType(i))
     {
      case TYPE_ZIP:  image = DirIcons.IconZip; isImage = true; break;
      case TYPE_IMAGE:image = DirIcons.IconPic; isImage = true; break;
      case TYPE_SOUND:image = DirIcons.IconWav; isImage = true; break;
      case TYPE_DOC:  image = DirIcons.IconDoc; break;
      case TYPE_EXE:  image = DirIcons.IconExe; break;
      default:        image = DirIcons.IconFile;
                              isOther = true;
                              break;
     }
  }


   // Version extended

   public int Displayed; // nombre d'affichs
   public int Masked;    // nombre de masqus

   public void dispDir(DirBox ilr, String Filter)
   {
    dispDir(ilr, Filter, 0);
   }

   public void dispDir(DirBox ilr, String Filter, int firstitem)
   {

     int i;
     String Node = null;
     String Ext = null;
     boolean Filtering;
     boolean accept = true;

     dirbox = ilr;
     int lastselected = dirbox.getSelectedIndex();

     if(!getList())
     {
       dirbox.clear(0);
       dirbox.setVisible(false);
       dirbox.setVisible(true);
       updated = true;
       return;
     }

     if((Filter == null) || zipMode) Filtering = false;
     else
     {
      Node       = Path.splitNode(Filter);
      Ext        = Path.getExtension(Filter);
      Filtering  = (Filter.length() > 0);
     }

     Displayed = 0;
     Masked = 0;

     // Initialisation des tableaux
     dirbox.clear(Contenu.length);
     dirbox.setModel(Contenu);
     dirbox.setImage(DirIcons.IconDir);

     for(i = MAXINDISKS; i < MAXINDIRS; i++)
        dirbox.addItemDir(Idx[i]);

     for(i = MAXINDIRS; i < MAXINLIST; i++)
     {
      setCategory(i);

      accept = true;
      if(maskImage && !isImage) accept = false;
      if(maskOther && isOther ) accept = false;


      int no = Idx[i];

      if(Filtering)
      {
         String s = Contenu[no];
         String CNode;
         String CExt;

         int  lio = s.indexOf('.');
         if(lio == -1) { CNode = s; CExt = ""; }
         else
          {
            CNode = s.substring(0, lio);
            CExt  = s.substring(lio + 1);
          }

        accept = (Node.equals("*") || Node.equalsIgnoreCase(CNode)) &&
                 (Ext.equals("*")  || Ext.equalsIgnoreCase(CExt));
      }

      if(accept)
      {
       dirbox.addItemFile(image, no, EntrySizes.elementAt(no), EntryTimes.elementAt(no));
       Displayed++;
      }
      else Masked++;
     } // for

     dirbox.setVisible(false);
     dirbox.setTopRow(firstitem);
     dirbox.setLastSelected(lastselected);
     dirbox.setVisible(true);
     updated = true;

  } // Disp List



};  // end of  class Directory
