/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

// Slider horizontal

public class HSlider extends Slider
{
    /**
     * Constructs a default HSlider.
     * The size is calculated as 260 x 24 with freq of 1 and 256 ticks
     * The ticks are drawn on both sides of the gauge with a frequency of 1.
     * The minimum value is 0. The maximum value is 255. 
     * The border is shown.
     */
    public HSlider()
    {
        super(); 
        this.thumb	= new HSliderThumbBoth();
        width		= BORDER_X * 2 + freq * (max + 1);
        height      = BORDER_Y * 2 + 4;
    }

    /** 
      * Dimensions auto
      * Construit le slider horizontal avec une position donne
      */
    protected HSlider(int x, int y)
    {
       this();
       setBounds(x, y, width, height);  
    }    

    protected HSlider(int x, int y, int startval)
    {
       this();
       setValue(startval);
       setBounds(x, y, width, height);  
    }    

    /**
      * Surcharg pour mettre  jour les variables de taille
      */

    public void setBounds(int x, int y, int w, int h)
    {
      super.setBounds(x, y, w, h);
      width = w;
      height = h;
    }    

    /** Sets the current slider tick mark style */
    public void setTickStyle(int style)
    {
    	if (this.style != style)
    	{
	        super.setTickStyle(style);
	        switch (style)
	        {
	            case TICK_TOP :    thumb = new HSliderThumbTop(); break;
	            case TICK_BOTTOM : thumb = new HSliderThumbBot(); break;
	            default :          thumb = new HSliderThumbBoth();  break;
	        }
	        repaint();
    	}
    }

	/**
	 * Tells this component that it has been added to a container.
	 * It has been overridden here to handle registering event listeners.
	 */
	public synchronized void addNotify()
	{
		super.addNotify();

		//Hook up listeners
		if (mouse == null)
		{
			mouse = new Mouse();
			addMouseListener(mouse);
		}
		if (mouseMotion == null)
		{
			mouseMotion = new MouseMtn();
			addMouseMotionListener(mouseMotion);
		}
	}

	/**
	 * It has been overridden here to handle registering event listeners.
	 */
	public synchronized void removeNotify()
	{
		//Unhook listeners
		if (mouse != null)
		{
			removeMouseListener(mouse);
			mouse = null;
		}
		if (mouseMotion != null)
		{
			removeMouseMotionListener(mouseMotion);
			mouseMotion = null;
		}

		super.removeNotify();
	}

	/** Mouse Event handling innerclass */
	class Mouse extends java.awt.event.MouseAdapter implements java.io.Serializable
	{
		/** Handles Mouse Pressed events */
		public void mousePressed(MouseEvent e)
		{
			moveThumb( e.getX(), true );
		}
	}

	/** MouseMotion Event handling innerclass */
	class MouseMtn extends java.awt.event.MouseMotionAdapter implements java.io.Serializable
	{
		/** Handles Mouse Dragged events */
		public void mouseDragged(MouseEvent e)
		{
			moveThumb( e.getX(), false );
		}
	}


    /**
      Dfinit taille
      Cre le tableau de ticks
      */
      
    protected void do_reshape(int w, int h)
    {
        int hb = BORDER_X;
        int vb = BORDER_Y;

        // Rduit les bords au quart de la barre si trop petite
        if (w < hb)  hb = w / 4;
        if (h < vb)  vb = h / 4;

        int xl = hb;           // Bord intrieur gauche
        int xr = w - hb - 1;   // Bord intrieur droit
        int yt = vb + 1;
        int yb = h - vb - 1;

        int n = ((max - min) / freq) + 1;  // Nombre de ticks

        tick = new SliderTick[n];  // Cration tableau de ticks

        // Largeur d'un tick
        int hs = tickWidth;
        int x;   // Position horizontale du tick sur la barre

        //Disp.it("reshape nombre de ticks " + n + " largeur " + hs);

        for (int i = 0; i < n; i++)
        {
            x = i * hs;  // Position sur la barre
            tick[i] = new SliderTick(yt, yb, xl + x, x);
        }

        // Ajuste la taille du curseur
        // Largeur: Moiti de la largeur
        thumb.resize(5, yb - yt - TICK_HEIGHT - 1);
    }

	/** 
	   Paints this component using the given graphics context 
	 */
	public void update(Graphics g) { paint(g); } 

    public void paint(Graphics g)
    {
        if(!notified) return;
        
    	super.paint(g);    // Ajuste les dimensions et appelle do_reshape

        if (tick.length == 0) return;

        int w = width - 1;
        int h = height - 1;
        int x0, x1, y;
        boolean end;

        final int xl = 1;
        final int yt = 1;
        int xr = xl + w - 2;
        int yb = yt + h - 2;

        //g.setColor(Color.black);
        //g.fillRect(0, 0, w, h); 
        // Cadre noir
        g.setColor(Color.black);
        g.drawRect(0, 0, w, h);

        // Remplit le fond  la couleur donne
        g.setColor(background);
        g.fillRect(2, 2, w - 4, h - 4);

        if(borderStyle == BORDER_RAISED)
         {
         g.setColor(background.brighter());
         g.drawLine(xl, yt, xr, yt);
         g.drawLine(xl, yt, xl, yb);
         g.drawLine(xl + 1, yt + 1, xr - 1, yt + 1);
         g.drawLine(xl + 1, yt + 1, xl + 1 , yb - 1);
         
         g.setColor(background.darker());
         g.drawLine(xl + 1, yb, xr, yb);
         g.drawLine(xr, yt + 1, xr, yb);
         g.drawLine(xl + 2, yb - 1, xr - 1, yb - 1);
         g.drawLine(xr -1 , yt + 2, xr - 1, yb - 1);
         }
        else if(borderStyle == BORDER_LOWERED)
         {
         g.setColor(background.darker());
         g.drawLine(xl, yt, xr, yt);
         g.drawLine(xl, yt, xl, yb);
         g.drawLine(xl + 1, yt + 1, xr - 1, yt + 1);
         g.drawLine(xl + 1, yt + 1, xl + 1 , yb - 1);
         
         g.setColor(background.brighter());
         g.drawLine(xl + 1, yb, xr, yb);
         g.drawLine(xr, yt + 1, xr, yb);
         g.drawLine(xl + 2, yb - 1, xr - 1, yb - 1);
         g.drawLine(xr -1 , yt + 2, xr - 1, yb - 1);
        }

        g.setColor(Color.black);   // Cadre et ticks en noir
        
        // Place les ticks
        if(style > 0)     // 0 = NONE
          for (int i = 0; i < tick.length; i += tickStep)
           {
            // end true quand on est aux extrmits soit 0 ou 255
            // dans ce cas les barres sont plus longues
            end = (i == 0) || (i == (tick.length - 1));
            SliderTick t = tick[i];
            if (style == TICK_TOP || style == TICK_BOTH)
                g.drawLine(t.x, t.yt + (end ? 0 : 1), t.x, t.yt + TICK_HEIGHT);

            if (style == TICK_BOTTOM || style == TICK_BOTH)
                g.drawLine(t.x, t.yb - TICK_HEIGHT, t.x, t.yb - (end ? 0 : 1));
          }

        SliderTick t = tick[0];     // Cre un tick sur la base du premier

        // Dessine la barre 

        //y  = (t.yb + t.yt) / 2;          // Hauteur du curseur
        y = BORDER_Y + 1; 
        x0 = t.x - 5;                      // Position premier - 5 pixels
        x1 = tick[tick.length - 1].x + 5;  // Position dernier + 5 


        //Disp.it("barre xl " + x0 + " y " + y + " xr " + x1 );
        g.drawLine(x0, y, x1, y);     
        g.drawLine(x0, y + 1, x1, y + 1);     

        g.setColor(Color.gray);
        g.drawLine(x1 + 1, y - 1, x0 - 1, y - 1);
        g.drawLine(x0 - 1, y - 1, x0 - 1, y + 1);

        g.setColor(Color.lightGray);
//        g.drawLine(x0, y + 1, x1 + 1, y + 1);
        g.drawLine(x1 + 1, y + 1, x1 + 1, y);

        g.setColor(Color.white);
        g.drawLine(x0 - 1, y + 2, x1 + 2, y + 2);
        g.drawLine(x1 + 2, y + 2, x1 + 2, y - 1);
       
        thumb.draw(g, tick[curPos]);

        prevPos = curPos;
    }


	/** Length of the gauge ticks in pixels */
    protected static final int TICK_HEIGHT  = 4;

    /** The command name of the action event fired by this component */
 	protected String actionCommand = "HSliderMoved";

    private HSliderThumb thumb;
    private Mouse		mouse			= null;
    private MouseMtn	mouseMotion		= null;

}

abstract class HSliderThumb implements java.io.Serializable
{
    void resize(int width, int height)
    {
        x = (this.width = width) / 2;
        y = (this.height = height) - HSlider.TICK_HEIGHT - 1 - 1;
    }

    abstract void draw(Graphics g, SliderTick t);

    // Trace une ligne relative  tick.x, tick.yt, tick.yb

    protected void draw(int x0, int y0, int x1, int y1)
    {
        g.drawLine(t.x + x0, t.yt + y0 + HSlider.TICK_HEIGHT + 1,
                   t.x + x1, t.yt + y1 + HSlider.TICK_HEIGHT + 1);
    }

    protected void initDraw(Graphics g, SliderTick t)
    {
        this.g = g;
        this.t = t;

        g.setColor(Color.lightGray);
        g.fillRect(t.x - x + 1, t.yt + 2 + HSlider.TICK_HEIGHT + 1, width - 2, y - 2);

        g.setColor(Color.white);
    }

/*
    void clip(Graphics g, SliderTick t)
    {
        g.clipRect(t.x - width / 2, t.yt, width + 1, height + 1);
    }
*/

    protected int x;
    protected int y;
    protected int width;
    protected int height;
    transient protected Graphics g;
    transient protected SliderTick t;
}


class HSliderThumbBoth extends HSliderThumb
{
    // appelle les fonctions draw drives
    void draw(Graphics g, SliderTick t)
    {
        super.initDraw(g, t); // calcule les rectangles

        // Maintenant couleur blanche
        // x = largeur / 2
        // y = hauteur - TICK_HEIGHT
        draw(-x, y, -x, 1);
        draw(-x, 1, x, 1);

        g.setColor(Color.black);
        draw(-x, y, x, y);
        draw(x, y, x, 1);

        g.setColor(Color.gray);
        draw(1 - x, y - 1, x - 1, y - 1);
        draw(x - 1, y - 1, x - 1, 2);
    }
}


class HSliderThumbTop extends HSliderThumb
{
    void draw(Graphics g, SliderTick t)
    {
        super.initDraw(g, t);

        int a = y / 5;

        draw(-x, y, -x, a);
        draw(-x, a, 0, 1);

        g.setColor(Color.black);
        draw(0, 1, x, a);
        draw(x, a, x, y);
        draw(x, y, -x, y);

        g.setColor(Color.gray);
        draw(0, 2, x - 1, a);
        draw(x - 1, a, x - 1, y - 1);
        draw(x - 1, y - 1, 1 - x, y - 1);
    }
}


class HSliderThumbBot extends HSliderThumb
{
    void draw(Graphics g, SliderTick t)
    {
        super.initDraw(g, t);

        int a = height - HSlider.TICK_HEIGHT - 1 - 1 - y / 5;

        draw(x, 1, -x, 1);
        draw(-x, 1, -x, a);
        draw(-x, a, 0, y - 1);

        g.setColor(Color.black);
        draw(0, y, x, a);
        draw(x, a, x, 1);

        g.setColor(Color.gray);
        draw(0, y - 1, x - 1, a);
        draw(x - 1, a, x - 1 , 2);
    }
}

