/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.image.*;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.event.*;
import common.*;

/**
   ImageViewer component.

   - getImage retourne une image  partir de l'image
     dja charge ou  partir du disque
   - setImage dfinit l'image du composant
   - xxxModel sont des fonctions internes

   Les paramtre sont: positionnement et mode d'agrandissement.
   On ne distingue ces modes qu'en position centre.
   Par dfaut l'image est scaled, avec CENTERED et PROPORTIONAL
   l'image est normale ou scaled quand elle est trop grande
 */

public class ImageViewer extends Panel implements MouseListener
{

    public static final int NORMAL = 0; /** drawn in the upper left corner. */
    public static final int TEXTURE = 1; /* scaled en texture */
    public static final int CENTERED = 2; /* centered */
    public static final int TILED = 4;     /* tiled */


	protected int imageStyle;    /** Determines how to draw the image.*/

    /** mode is normal or scaled */
    public static final int NONE = 0;   // texture
    public static final int SCALED = 2;  // agrandie
    //public static final int SCALE_NONE = 0;

    public int scaleMode;

    public boolean newSize = false; // taille de fentre change

    Dimension oldSize;
    Dimension dim;

    public int originW;
    public int originH;
    public int imageW;
    public int imageH;


    transient public Image image;/** Image that this viewer is displaying. */
    protected String fileName;/** Name of file, if any, associated with this image. */
    protected URL url;   /** URL of the image being displayed. */

    public String name;  // Nom de l'image

    /**
     * Create default image viewer.
     */

    public ImageViewer()
    {
     image    = null;
     fileName = null;
     name     = null;
     url      = null;
	 imageStyle	= CENTERED;
	 scaleMode  = NONE;
	 oldSize    = new Dimension(0, 0);

	 addMouseListener(this);
	}

    /**
    * Create image viewer with filename.
    */


	public ImageViewer(String str)
	{
	  this();
	  setImage(str);
	}

   /**
    * Create image viewer with URL.
    */

	public ImageViewer(URL url)
	{
	 this();
  	 setURL(url);
	}

    /**
       Create image viewer with image.
       The specified image is used as the image source
     */

	public ImageViewer(Image img)
	{
	  this();
	  setImage(img);
	}


    void error(String s) { System.out.println(s); }

    /** Donne un nom */
    public void nameIt(String n) { name = n; }


    /**
     * Sets the new panel image style.
     * @param newStyle the new panel image style, one of
     */
	public void setStyle(int newStyle) 	{ imageStyle = newStyle; }

    /**
       Set the mode:
       SCALE_TO_FIT   map the panel with the image
       SCALE_REGULAR  resize the image inside the panel's limits
      */
   public void setScaleMode(int sm) { scaleMode = sm; }


  /**  Donne la taille de l'image  */

  public Dimension getDimension() { return new Dimension(imageW, imageH); }


   /**
   * Gets the current panel image style.
   */
	public int getStyle() {	return imageStyle;	}

   /**
      retourn l'image
   */
    public Image getImage() {  return image; }
    public Image getModel() { return model; }

   /**
     Load the image
     Charge simplement l'image
   */

   public Image loadImage(String path)
   {
    return loadImage(getToolkit().getImage(path));
   }

  public Image loadImage(Image img)
  {
    MediaTracker tracker = new MediaTracker(this);
    tracker.addImage(img, 0);
    //error("loadimage");
    try
     {
      tracker.waitForAll();
      return img;
     }
    catch(InterruptedException e) { }

    return null;
  }



  /**
      Gestion de l'image originale
      Charge une image en mmoire et affecte  model
      Model ne scale jamais l'image
  */

   transient protected Image model;
   transient protected Image oldModel;

   public void setModel(Image img)
	{
     model = loadImage(img);
     oldModel = model;

     imageW  = model.getWidth(this);
     imageH  = model.getHeight(this);
     originW = imageW;
     originH = imageH;
     //System.out.println(imageW + " " + imageH);
	}


	public void setModel(String path)
	{
     fileName = path;
     setModel(getToolkit().getImage(path));
	}





  static public Image resize(Image model, int originW, int originH, int ivw, int ivh)
   {
     int H = ivh;
     int W = (int) ( (float) originW * (float) ivh / (float) originH);

     if(W > ivw)
      {
       W = ivw;
       H = (int) ((float) originH * (float) ivw / (float) originW);
      }

     return model.getScaledInstance(W, H, Image.SCALE_DEFAULT);
   }

  public Image resize(Image model)
   {

     int ivw = getWidth();
     int ivh = getHeight();

     imageH = ivh;
     imageW = (int) ( (float) originW * (float) ivh / (float) originH);

     if(imageW > ivw)
      {
       imageW = ivw;
       imageH = (int) ((float) originH * (float) ivw / (float) originW);
      }

     return model.getScaledInstance(imageW, imageH, Image.SCALE_DEFAULT);
   }

    /**
       Substitut  setImage
       qui adapte la taille de l'image  celle du panneau
      */

   /*
   public void setImageResized(String path)
    {
      scaleMode = SCALE_REGULAR;
      setModel(path);
      image = resize(model);
   }

    public void setImageMapped(String path)
    {
       imageStyle = SCALE_TO_FIT;
       setModel(path);
       image = model;
      }
  */

   // Need resize?
    boolean scaleNeeded()
    {
     return (originW >= getWidth() || originH >= getHeight());
    }


  /**
     Set or change the image.
     Scale si elle est trop grande pour le panneau
  */

    public void setImage(String path)
    {
       setModel(path);

       if(scaleNeeded()) image = resize(model);
       else  image = model;
    }

	public void setImage(Image img)
	{
      setModel(img);

      if(scaleNeeded()) image = resize(model);
      else  image = model;

      fileName = null;
	}

    /** Indique si l'image est agrandie ou non */
    public void setImage(String path, int scaleflag)
    {
     scaleMode = scaleflag;
     setModel(path);

     switch(scaleflag)
     {
      case SCALED:
            image = resize(model);
            imageStyle = CENTERED;

      default:   // taille normale
            image = model;
     }

     return;
    }

	/**
     Specify or change the image filename
    */
/*
	public void setFileName(String str)
	{
     fileName = str;
     try
     {
       if(fileName != null && fileName != "")
	           setURL(new URL(fileName));
       else
	           setURL(null);
     } catch(MalformedURLException m) {}
	}
*/
	/**
     * Obtain the filename associated with the current image.
     *
     * @return the name of the file, if any, associated with this image.  If
     * no file is associated with this image, returns null
     */
	public String getFileName()	{   return fileName;}

    /**
     * Specify or change the image URL.
     * @param aUrl the URL of the image to be displayed
     */
	public void setURL(URL aUrl)
	{
	  Image loadedImage = null;

	  url = aUrl;
	  fileName = null;

	  if(url != null)  loadedImage = getToolkit().getImage(url);
	  setImage(loadedImage);
	  repaint();
	}

   /**
     * Obtain the URL associated with the current image.  If the image
     * was specified by file name, or URL, it will have a URL which
     * indicates its source.  Images created using the constructor
     * with an Image parameter will have no associated URL.
     */
	public URL getURL()	{  return url;	}



    /** Efface le fond */
	public void clear()
	{
	  Graphics g = getGraphics();

	  Dimension d = getSize();

	  g.setColor(getBackground());
	  g.fillRect(0, 0, d.width, d.height);
	}


/**
 * Paints this component using the given graphics context.
 */

public void update(Graphics g) { paint(g); }


public void paint(Graphics g)
{
	super.paint(g);

    //System.out.println("paint " + counter++);

	dim = getSize();
	int panelW = dim.width;
	int panelH = dim.height;

	if ((imageW + imageH) < 512)
       g.clearRect(0, 0, panelW, panelH);

	if (image == null) return;

    if(model != null)
        newSize = (!model.equals(oldModel)) || (!oldSize.equals(dim));

	switch(imageStyle)
	 {
		case CENTERED:

		default:
		    // Pour mettre  jour quand on change la taille de fentre
	    if(scaleMode == SCALED)
          {
	       if(newSize) image = resize(model);
          }
        else  // normal centr sauf si taille...
          {
           if(newSize) // non scaled, taille change
             {
               if(originW >= panelW || originH >= panelH)
                 image = resize(model); // image originale > fentre
               else // actuellement elle tient dans la fentre
                 if(imageW != originW || imageH != originH) // mais pas avant
                  {
                    image  = model;
                    imageW = originW;
                    imageH = originH;
                  }
              }
          }

         int x = (panelW - imageW) / 2;
         int y = (panelH - imageH) / 2;
	     g.drawImage(image, x, y, imageW, imageH, this);
		 break;

		case TILED:
			//Calculate number of images drawn horizontally and remainders
			int numHImages = panelW / imageW;
			if (panelW % imageW != 0) numHImages++;
			//Calculate number of images drawn vertically
			int numVImages = panelH / imageH;
			if (panelH % imageH != 0) numVImages++;
			int h;
			int v = 0;
			for (int vCount = 0;vCount < numVImages;vCount++)
			{
				h = 0;
				for (int hCount = 0;hCount < numHImages;hCount++)
				{
					g.drawImage(image, h, v, imageW, imageH, this);
					h += imageW;//Increment to next column
				}
				v += imageH; //Increment to next row
			}

			break;
		case TEXTURE:
			g.drawImage(image, 0, 0, panelW, panelH, this);
			break;
		case NORMAL:
			g.drawImage(image, 0, 0, this);
			break;
   }//switch

   oldSize = dim;
 }


    /**
	 * Returns the recommended dimensions to properly display this component.
     * @return  If no image has been loaded, a dimension of 10 by 10 is returned.
     *          If an image has been loaded, the height and width of the image
     *          is returned.
	 */
	public Dimension getPreferredSize()
	{
		if (image != null)
	        return (new Dimension(image.getWidth(this), image.getHeight(this)));
		else
	        return new Dimension(10, 10);
	}


    /**
	 * Returns the minimum dimensions to properly display this component.
	 */

	public Dimension getMinimumSize() { return getPreferredSize();  }


	/**
	 * Sends an action performed event to any action listeners, as needed.
	 */

    ActionListener imgAL;

	public void addActionListener(ActionListener l) {
	  imgAL = AWTEventMulticaster.add(imgAL, l);
	}

	public void removeActionListener(ActionListener l) {
	  imgAL = AWTEventMulticaster.remove(imgAL, l);
	}

    public void sendAction(String command)
     {
      if (imgAL != null)
	    imgAL.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
     }

	protected void sendResizeEvent()
	 {
	  sendAction("Resized");
	 }



	/**
	 * This is the Mouse Event handler
	 */
	public void mouseClicked(MouseEvent e) {}
	 {
//	   sendAction("full");
	 }


	public void mousePressed(MouseEvent e)
	{
			/*
			if (e.isPopupTrigger())  // right button sur PC
			{
 		       //pop.show(parent, x, y);
 		       sendActionEvent("enter");
 		       return;
            }
            */
	}
	public void mouseReleased(MouseEvent e)
	 {
	   sendAction("full");
	   /*
    	if (e.isPopupTrigger())  // right button sur PC
		{
		   //sendAction("Enter");
		   //requestFocus();

 		   //   pop.show(getParent().getParent().getParent(), x, y);

 		   return;
        }
       */
 	}

	public void mouseExited(MouseEvent e) {	}
	public void mouseEntered(MouseEvent e) {}

}
