/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import common.*;

public class PanelArchive extends PanelGenerator
{

  DCheckbox checkRecurse;
  DCheckbox checkInclude;


  public PanelArchive(Frame f)
   {
    super(f);
    setTitle("ARCHIVER");
    setAction("Make an archive of selected items");

    DBPanel options = new DBPanel(Bevel.LINE);
    options.setLabel("Options");
    add(options, BorderLayout.CENTER);

    checkRecurse = new DCheckbox("Recurse sub-directories", Preferences.ArcRecurse);
    checkRecurse.setBounds(32, 32, 200, 24);
    checkRecurse.addItemListener(this);
    options.add(checkRecurse);

    checkInclude = new DCheckbox("Include full paths", Preferences.ArcInclude);
    checkInclude.setBounds(32, 64, 200, 24);
    checkInclude.addItemListener(this);
    options.add(checkInclude);

    htmlName = "";

   }


   public void init(DirManager src, DirManager tgt, boolean b)
   {
    super.init(src, tgt, b);

    switch(Mode)
    {
     case SINGLE: setAction("Archive the selected file");
                  break;
     case FOLDER: setAction("Archive content of the selected folder");
                  break;
     case TAGGED: setAction("Archive the tagged list");
                  break;
     default: break;
    }

   }

   public boolean archive(DirManager source)
   {
    //debug(""); debug("enter archiving");

    String path = getDir();    // Chemin archive = Dst Path au dpart
    String name = getFilename();    // Nom archive
    String origine = getSource();   // Nom fichier ou TAGGED LIST
    String archive;    // Chemin complet avec nom archive


    //debug("arc  " + archive);
    Archiver.setMessage("Error");

    if(Mode == NONE)
    {
      dispTip("Nothing to do! Select files/dir in first panel, please...");
      return false;
    }

    if(name == null || name.length() == 0)
    {
      dispTip("Give a name to the archive, please!");
      return false;
    }

    if(Path.hasExtension(name))
    {
     if(Directory.isGZip(name))
     {
      if (Mode != SINGLE)
       {
        dispTip("GZip require a single file"); return false;
       }
     }
     if(!Directory.isArchive(name))
      {
       dispTip("Give a valid archive extension or none (default is zip)");
       return false;
      }
    }
    else
    {
     name = name + ".zip";
    }

    setFilename(name);
    archive  = Path.merge(path, name);

    // Test si archive existe dja
    boolean replacing = (new File(path, name)).exists();

    if(replacing)
     {
       DialogCYN cd = new DialogCYN(frame, "Erasing the current " + name.toUpperCase() + " archive, continue?");
       cd.setVisible(true);
       if(cd.answer != cd.YES) return false;
     }


    boolean result = false;

    Cursor cursor = getCursor();
    setCursor(new Cursor(Cursor.WAIT_CURSOR));


    if(Mode == SINGLE)
    {
     message("Archiving " + origine + " into " + archive);

     result = Archiver.archiveAnyFile(origine, archive);

     if(result) message("Archive completed");
           else message(Archiver.getMessage());

     setCursor(cursor);
     return true;
    }

    String fics[] = source.getSelectedItems();
    String srcpath = null;
    String srcdir = source.getDir();

    //debug("fics len " + fics.length);

    if(Mode == FOLDER)
    {

     String fic = fics[0];
     message("Archiving " + fic + "'s content into "+ archive);

     // Je veux inclure le nom du chemin
     if (Preferences.ArcInclude)
     {
       srcpath = srcdir;
     }
     else
     {
       srcpath = Path.merge(srcdir, fic);
       fics = (new File(srcdir, fic)).list();
     }

    }


    if(Mode == TAGGED)
    {
     message("Archiving " + origine + " into "+ archive);
     srcpath = srcdir;
    }

    // Si la liste contient des lments, on archive
    if(fics.length > 0)
    {
     int fr = Archiver.archive(srcpath, fics, archive);
     message("Archived " + fr + " files");

     if(Archiver.error)
       (new DialogMessage(frame, "Error in archiving")).setVisible(true);
    }
    else dispTip("Nothing to do");

   setCursor(cursor);
   return true;

  }



   public void itemStateChanged(ItemEvent event)
    {
  	 Object object = event.getSource();

     if (object == checkRecurse)
	  Preferences.ArcRecurse = checkRecurse.getState();
     else if (object == checkInclude)
	  Preferences.ArcInclude = checkInclude.getState();
//     else if (object == checkStore)
//	  Preferences.StoreOnly = checkStore.getState();
   }

}

