/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import common.*;

public class PanelGenerator extends Panel implements ActionListener, ItemListener, Messages, KeyListener
{
  // Archiver

  final short NONE = 0;
  final short SINGLE = 1;
  final short TAGGED = 2;
  final short FOLDER = 3;


  Frame frame;

  TextField fieldDir;
  TextField fieldName;
  DButton buttonMake;
  DButton buttonCancel;
  DLabel labelTitle;
  DLabel labelSource;
  Label labelName;
  Label labelPath;

  public String htmlFile = null; // le fichier html avec chemin
  public String htmlName = "generated.htm"; // le fichier html
  public String htmlDir = "";



 public PanelGenerator(Frame f)
 {

  super();
  setVisible(false);
  setLayout(new BorderLayout());

  frame = f;

  int fieldSize = 24;
  if(!OS.isWindows()) fieldSize = 30;

  EmptyPanel panelFields = new EmptyPanel();
  panelFields.setBounds(0, 0, 450, 270);
  add(panelFields, BorderLayout.NORTH);

  Font font = new Font("Dialog", Font.BOLD, 16);

  labelTitle = new DLabel();
  labelTitle.setBounds(64, 16, 320, 32);
  labelTitle.setFont(new Font("SansSerif", Font.BOLD, 24));
  labelTitle.setTextColor(f.getBackground().darker());
  panelFields.add(labelTitle);

  labelSource = new DLabel();
  labelSource.setBounds(16,60,420,24);
  labelSource.setFont(font);
  panelFields.add(labelSource);

  labelPath = new Label("Path");
  labelPath.setBounds(16,92,128,20);
  panelFields.add(labelPath);

  fieldDir = new TextField();
  fieldDir.setBounds(16,115,360,fieldSize);
  fieldDir.addKeyListener(this);
  panelFields.add(fieldDir);

  labelName = new Label("Name");
  labelName.setBounds(16,148,128,20);
  panelFields.add(labelName);

  fieldName = new TextField();
  fieldName.setBounds(16, 171, 256, fieldSize);
  fieldName.addKeyListener(this);
  panelFields.add(fieldName);

  buttonCancel = new DButton("Cancel", true);
  buttonCancel.setBounds(100, 216, 100, 32);
  buttonCancel.addActionListener(this);
  buttonCancel.setTip("Return to the target panel");
  panelFields.add(buttonCancel);

  buttonMake = new DButton("Make", true);
  buttonMake.setBounds(220, 216,100, 32);
  buttonMake.addActionListener(this);
  panelFields.add(buttonMake);

}


 public void setAction(String action) { buttonMake.setTip(action); }
  /** Quels fichiers archivs */
 public  void setTitle(String title)  { labelTitle.setText(title); }
  /** Bouton enabled quand les champs sont remplis */
 public  void setMakeEnabled(boolean f) { buttonMake.setEnabled(f); }

 public  void setSource(String p) { labelSource.setText(p); }
 public String getSource() { return labelSource.getText(); }

 public  void setDir(String p)
  {
    fieldDir.setText(p);
    htmlDir = p;
  }

 public  String getDir(String d)
   {
    if ((htmlDir == null) || (htmlDir == "")) return d;
    return getDir();
   }
 public  String getDir() { return htmlDir; }


 public  void makeFile()
  {
   htmlFile = Path.merge(htmlDir, htmlName);
  }

 public  String getFile() { return htmlFile; }

 public  void setFilename(String name)
  {
   htmlName = name;
   fieldName.setText(name);
  }

 public  String getFilename() { return fieldName.getText(); }


  int Mode = NONE;

  public void init(DirManager source, DirManager target, boolean selectall)
  {
    String origin = null;
    String fic = null;
    String dir = source.getDir();

    int[] se = source.getSelectedIndexes();

    Mode = NONE;
    buttonMake.setEnabled(true);

    switch(se.length)
    {

     case 1:
            fic = source.getSelected();
            origin = Path.merge(dir, fic);
            File f = new File(dir, fic);
            if(f.isDirectory())  Mode = FOLDER;
            else                 Mode = SINGLE;
            break;

     case 0:

            if (selectall)
            {
              source.selectAll();
              // continuer sur case suivant
            }
            else
            {
             buttonMake.setEnabled(false);
             origin = "Select some file please";
             dispTip(origin);
             break;
            }


     default:
            Mode = TAGGED;
            origin = dir + File.separator + "[TAGGED LIST]";
            break;
    }


  setSource(origin);
  setDir(source.getDir()); // dir, name, path
  setFilename(htmlName); // dir, name, path
  makeFile(); // dir, name, path

}



 //------------------------------ EVENT HANDLER

 public  String message = "";


 public void message(String s) { message = s; sourceActionEvent(MESSAGE); };
 public void dispTip(String s) { message = s; sourceActionEvent(TIP); };

 public void actionPerformed(ActionEvent event)
  {
      Object object = event.getSource();
      if (object == buttonMake)
        {
  	    htmlDir  = fieldDir.getText();
  	    htmlName = fieldName.getText();
  	    makeFile();
  	    sourceActionEvent(MAKE);
  	    return;
        }

      if (object == buttonCancel)
        {
  	    sourceActionEvent(CANCEL);
  	    return;
        }
 }


   /** External listener */
   ActionListener actionListener = null;

   /** Fire an action event to the listeners. */
public int action = 0;

 protected void sourceActionEvent(int a)
  {
    if (actionListener == null) return;
    action = a;
    actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null));
  }

   /**
      Allow to add an external listener,
      to send it action events
   */

  public void addActionListener(ActionListener l)
  {
     actionListener = AWTEventMulticaster.add(actionListener, l);
  }

  /**  surcharger */

  public void itemStateChanged(ItemEvent event) { }

  public void keyTyped(java.awt.event.KeyEvent event) {};
  public void keyReleased(java.awt.event.KeyEvent event) {};
  public void keyPressed(java.awt.event.KeyEvent event)
   {
     if(event.isControlDown() || event.isAltDown())  return;

     char c = event.getKeyChar();
     char newc = new Character(c).charValue();
     if (c != newc) event.consume();
     event.setKeyChar(newc);
   }

}
