/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.FontMetrics;
import java.awt.Image;

/**
   The TButton is a classical Text button component
 */
public class TButton extends DButtonBase implements java.io.Serializable
{
	protected Color	textColor			= null;
	protected Color	disabledTextColor	= null;
	Image icon;

    /** Constructs a default DirButton */
    public TButton() { this(""); }

    /**
     * Constructs a TButton with label
     */
	public TButton(String lab)
	{
	  super();
	  this.label = lab;
	  textColor = Color.black;
	  disabledTextColor = ColorUtils.lighten(textColor, (double) 1 / 2);
	}


    public TButton(Image icon, String lab)
    {
      this(lab);
      this.icon = icon;
    }

    /**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     *
     * @return a button that has a content area of 7 by 7 pixels.
     * @see java.awt.Component#getMinimumSize
	 */
    public Dimension getPreferredSize()
    {
    	Dimension defaultSize = super.getPreferredSize();

		return new Dimension(defaultSize.width + 5, defaultSize.height + 5);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     *
     * @return a button that has a content area of 3 by 3 pixels.
     * @see java.awt.Component#getMinimumSize
	 */
    public Dimension getMinimumSize()
    {
    	Dimension defaultSize = super.getPreferredSize();

	return new Dimension(defaultSize.width + 3, defaultSize.height + 3);
    }

	/**
	 * Maintains the buttonImage size and draws the
	 * button in the buttonImage offscreen image.
	 * @see ButtonBase#updateButtonImage
	 */
  protected void updateButtonImage()
  {
	super.updateButtonImage();
	Graphics g		= buttonImage.getGraphics();
	Dimension s		= getSize();

        int w = s.width;
        int h = s.height;

        g.setFont(super.getFont());
        FontMetrics fm = g.getFontMetrics();
        int lh = fm.getHeight() + 4;
        int lw = fm.stringWidth(label) + 6;

        int xLab = (w - lw) / 2 + 3;
        int yLab = (h - lh) / 2 + 3 + 13;

        if(icon != null)
        {
          int iWidth = icon.getWidth(this);
          int iHeight = h - 8;
          xLab = h + (w - h - lw) / 2;
          g.drawImage(icon, 4, (h - iHeight) / 2, iWidth, iHeight, this);
        }

        if(isEnabled()) {  g.setColor(textColor); }
        else       {       g.setColor(disabledTextColor);  }

        g.drawString(label, xLab, yLab);
		if (g != null)  g.dispose();
  }

    /**
     * Is the given bevel size valid for this button.
     * @param i the given bevel size
     * @return true if the given bevel size is acceptable, false if not.
     */

    protected boolean isValidBevelSize(int i)
    {
        Dimension s = getSize();

        int temp = i * 2 + 4;

        return !(i < 0 || s.width < temp || s.height < temp);
    }



}
