/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

public class TextEditor extends Panel implements KeyListener, ActionListener
{
   final int CANCEL = 0;
   final int YES    = 1;
   final int NO     = 2;

   public String TextPath = null;
   public String TextName = null;
   public String TextDir = null;
   public String UpdateDir = null;

   public String m_dirName;
   public String m_fileName;
   public String m_pathName;

   FindReplace m_findReplace; // echange donnes avec FindDialog

   TextArea area;
   TextModel textModel;
   public String TextMessage = ""; // voir
   public static final String NoName = "+Editor";

   public boolean Editable = true;
   public boolean Edited = false;

   boolean wraparound;

   Frame frame = null;


   public TextEditor(Frame f)
    {
     super();

     setLayout(new GridLayout());
     frame = f;

     // Panel text
     int mode;
     wraparound = Preferences.wordWrap;

     if (wraparound)
       mode = TextArea.SCROLLBARS_VERTICAL_ONLY;
     else
       mode = TextArea.SCROLLBARS_BOTH;


     area = new TextArea("", 80, 16384,TextArea.SCROLLBARS_VERTICAL_ONLY);
     area.addKeyListener(this);
     add(area);

     textModel = new TextModel(area);
     m_findReplace = new FindReplace(area, textModel);

     TextName = NoName;
    }

   // public TextEditor() { this(null); }


   public void setFont(Font f)
   {
    super.setFont(f);
    area.setFont(f);
   }

   public void setBackground(Color c)
   {
    super.setBackground(c);
    area.setBackground(c);
   }

   public void setForeground(Color c)
   {
    super.setForeground(c);
    area.setForeground(c);
   }

   public void setStyle(Style s)
    {
     setBackground(s.background);
     setForeground(s.foreground);
     setFont(s.font);
    }


   public void setEditable(boolean state)
   {
     Editable = (state == false);
     area.setEditable(Editable);
   }

   public String getDir()
    {
      return m_dirName;
    }


/*
   public void setWrap(boolean yesno)
   {
    if(yesno == wraparound) return;

    wraparound = yesno;

    String text= area.getText();

    Color b = getBackground();
    Color f = getForeground();
    this.remove(area);

    if(wraparound)
    {
     dispTip("Wrap-around activated");
     int c = area.getColumns();
     int r = area.getRows();
     area = new TextArea(text, c, r, TextArea.SCROLLBARS_VERTICAL_ONLY);
    }
    else
    {
     dispTip("Wrap-around cancelled");
     area = new TextArea(text);
    }

     area.setBackground(b);
     area.setForeground(f);
     this.add(area);
     area.addNotify();
     area.invalidate();
     area.validate();
     area.repaint();
   }

   public void setWrap()
   {
    setWrap(!wraparound);
   }
*/

/*
   public void setWrap()
   {
    undo();

    FontMetrics fm = getFontMetrics(getFont());
    String text = "ae MZikn,";
    int tw = fm.stringWidth(text);
    int size;

    try
    {
     Dimension d = getSize();
     size = d.width / (tw / text.length());
     size -= 2;
    }
    catch(Exception e) { size = 40; }

    textModel.wrapIt(size);

   }
*/

 //-------------------------- Fonctions d'dition

  /*
     Quand on replace une chaine FindReplace.edited devient VRAI
     Quand on charge un fichier, il est dfinit FAUX
     Si CDWindow.Edited VRAI, find replace edited mis  VRAI
     sinon il peut tre VRAI ou FAUX
  */

  public void setEdited(boolean e)
   {
    Edited = e;
    m_findReplace.setEdited(e);
   }


  public  boolean isEdited()
   {
    if(Edited) return true;
    if(m_findReplace.isEdited()) return true;
    return false;
   }


  public void find()
  {
      m_findReplace.setSelection(textModel);

      FindDialog fd = new FindDialog(frame, "Find", false, m_findReplace);

      try { fd.setVisible(true); }
      catch(NullPointerException ne) { ne.printStackTrace(); }

 }

 /*
    activ par F3
 */
public void findSelection()
    {
      if(textModel.isSelected())  // quelque chose de slect
      {
        m_findReplace.setFind(textModel.getSelectedText()); // envoie le texte
        m_findReplace.setIndex(textModel.getSelectionEnd()); // envoie la position
	    findString();  // continue la recherche
        return;
      }
      // rien de selectionn
      findAgain();
    }


/** Cherche encore, command par F3 */

  public void findAgain()
  {
      String str = m_findReplace.getFind();
      if(str == null || (str.length() == 0)) // pas de chaine donne
       {
        if(!m_findReplace.setSelection(textModel)) // pas de texte marqu?
        {
         find(); // alors boite de dialogue
         return;
        }
       }

      int index;
      if(textModel.isSelected()) index = textModel.getSelectionEnd();
      else                       index = textModel.getCaretPosition();

      m_findReplace.setIndex(index);
	  findString();
    }

    /** Recherche occurence par appel  FindIt de F..R..Engine */
  public void findString()
    {
       FindReplaceEngine fre = new FindReplaceEngine(m_findReplace);
       fre.FindIt();
       //debug("find return " + m_findReplace.isFoundIt());
       if(!m_findReplace.isFoundIt()) Tool.beep();
    }


   /** Replace */
 public  void replace()
 {
      m_findReplace.setSelection(textModel);

      try
      {
        ReplaceDialog rd = new ReplaceDialog(frame, "Replace", false, m_findReplace);
        rd.setVisible(true);
      } catch(Exception e) { e.printStackTrace(); }
 }



public void selectAll()
 {
    message("Entire text is selected");
    area.select(0, area.getText().length());
 }


 public void undo()
    {
      textModel.undo();
    }


 public void setTextInfo()
    {
      int t[] = textModel.getLineCol();
      TextMessage = "Line " + t[0] + " Column " + t[1] + "    Pos " + textModel.getCaretPosition() + "  Size " + textModel.size();
    }


//------------------------------ Disk

  public void setPath(String dir, String fic)
   {
     m_dirName = dir;
     m_fileName = fic;
     m_pathName = Path.merge(m_dirName, m_fileName);
   }

  public void changeName()
  {
    TextName  = m_fileName;
    TextDir   = m_dirName;
    TextPath  = m_pathName;
    UpdateDir = m_dirName;
    setEdited(false);
  }

  /** Nouveau texte */

  public boolean newText()  { return newText(""); }
  public boolean newText(String info)
   {
     if(savePrevious()) return false;
     area.setText(info);
     return true;
   }



  /** Chargement d'un fichier */

  public boolean open()
   {
   //debug("Open function " + m_dirName + m_fileName);
    if(m_fileName == null) return true; // ne pas enchainer
    if(m_pathName == null)
        { m_pathName = ""; m_dirName = ""; }

    //if(savePrevious()) return false;

    area.setText("");

    String text = Disk.loadText(m_dirName, m_fileName);
    int size = text.length();

    if(size > 0)
     {
      System.gc();
      //System.out.println("len before " + textModel.size());
      area.setText(text);
      //System.out.println("len after " + textModel.size());
      area.select(0,0);
      textModel.clearUndo();
      if(textModel.size() > 0)
      {
       message(m_fileName + " loaded, size " + size + " bytes");
       return true;
      }
      else
       message("Enable to display " + m_pathName);
     }
    else
     message("Enable to load " + m_pathName);

    return false;
   }



   public boolean close()
    {
     if(isEdited())
      {
       DialogCYN cd = new DialogCYN(frame, "Saving the text?");
       cd.setVisible(true);
       if(cd.answer == cd.YES) if(savePrevious()) return false;
       else if(cd.answer == cd.CANCEL) return false;
      }

     newText();
     return true;
   }


   public void insert()
   {
    FileDialog openDialog = new FileDialog(frame);
	openDialog.setMode(FileDialog.LOAD);
	openDialog.setTitle("Open");
	openDialog.setDirectory(m_dirName);
    openDialog.setVisible(true);

    insert(openDialog.getDirectory(), openDialog.getFile());
   }

   public void insert(String m_dirName, String m_fileName)
    {
	 if (m_fileName != null)
 	  {
	   if(m_dirName == null) m_dirName = "";
       int size = Disk.insert(m_dirName, m_fileName, textModel);
       if(size > 0)
        {
          message(m_fileName + " loaded and inserted (" + size + " bytes)");
          setEdited(true);
        }
       else
         message("Enable to load " + m_fileName + " from " + m_dirName);
     }
   }


   // Sauver avec un nouveau nom

 public boolean cancelled()
   {
    message("Cancelled...");
    return false;
   }

 public boolean saveAs()
  {
    if (TextName.compareTo(NoName) == 0) TextName = "";

     FileDialog saveDialog = new FileDialog(frame);
     saveDialog.setMode(FileDialog.SAVE);
     saveDialog.setTitle("Save");
     saveDialog.setFile(TextName);
     saveDialog.setDirectory(m_dirName);
     saveDialog.setVisible(true);

     String dir = saveDialog.getDirectory();
     String name = saveDialog.getFile();

     if (dir == null)        return cancelled();
     if (name == null)       return cancelled();
     if (name.length() == 0) return cancelled();

     setPath(dir, name);

     int size = Disk.saveProtect(m_dirName, m_fileName, Disk.TEXT, area.getText());
     if(size > 0)
       message(TextName + " saved as " + m_pathName + " " + size + " bytes");
     changeName();
     sendSource("modified");
     return true;
 }

     /** Retourne faux si cancel ou dit non sauv en bd */

public int saveBefore()
 {
     if(isEdited())
       {
         DialogCYN QD = new DialogCYN(frame, "Text not saved. Save it?");
         QD.setVisible(true);
	 if(QD.answer == QD.CANCEL) return CANCEL;
	 if(QD.answer == QD.YES)
         {
          if(!save()) return CANCEL;
          return YES;
         }
         // Si response n, retourne true
        }
    return NO;
 }


public boolean save()
 {
     boolean ret;

     //System.out.println("TextPath " + TextPath);
	 //System.out.println("textname " + TextName);
	 //System.out.println("noname " + NoName);

     if(TextName == null) return saveAs();
     if(TextName.length() == 0) return saveAs();
     if(TextName.compareTo(NoName) == 0) return saveAs();

     int size = Disk.saveProtect(TextDir, TextName, Disk.TEXT, area.getText());
     if(size > 0)
       {
         message(TextName + " saved, " + size + " bytes");
         setEdited(false);
         UpdateDir = TextDir;
         //System.out.println("sent " + UpdateDir);
         sendSource("modified");
         ret = true;
       }
       else
       {
         message("Error, not saved");
         ret = false;
       }

      return ret;
  }

   // Teste si edit
   // conserve les noms qui sont changs par SaveAs


 public boolean savePrevious()
 {
     if(!isEdited()) return false;

     String sname = m_fileName;
     String sdir  = m_dirName;

     boolean res = (saveBefore() == CANCEL);  // vrai si cancel

     setPath(sdir, sname); // restaure les noms
     return res;
 }



public boolean saveBlock()
  {

 	  String stext =  textModel.getSelectedText();
      if(stext.length() == 0)
       {
        dispTip("Select a block of text to save");
        Tool.beep(); return false;
       }

      FileDialog saveDialog = new FileDialog(frame);
      saveDialog.setMode(FileDialog.SAVE);
      saveDialog.setTitle("Save");
      saveDialog.setFile("");
      saveDialog.setDirectory(TextDir);
      saveDialog.setVisible(true);

      String BName = saveDialog.getFile();
      String BDir  = saveDialog.getDirectory();

      if (BName != null)
       {
          //debug(stext);
        int size = Disk.saveBlock(BDir, BName, stext);
        if(size > 0)
         {
           UpdateDir = BDir;
           message(BName + " saved into " + BDir + " " + size + " bytes");
           sendSource("modified");
           return true;
         }
        message("Error, not saved");
       }
      return false;
  }

   public void requestFocus()
   {
    area.requestFocus();
   }


   public void keyTyped(KeyEvent event) { }
   public void keyReleased(KeyEvent event) { }
   public void keyPressed(KeyEvent event)
    {
     int keypressed = event.getKeyCode();

     if (isVisible() == false) return;

     // Alt
     if(event.isAltDown())
       {
  		 switch(keypressed)
  		 {
  		  case KeyEvent.VK_W:
  		                      saveBlock();
  		                      event.consume();
                              return;
	      case KeyEvent.VK_I: if(Editable) insert();
                    	      event.consume();
                    	      return;
  		  default: sendKey(event); break;
         }
          return;
       }

        // Control
   	  if(event.isControlDown())
   		{
         switch(keypressed)
         {
           case KeyEvent.VK_F: find();
                               event.consume(); return;
           case KeyEvent.VK_D: setTextInfo();
                               dispTip(TextMessage);
                               event.consume(); return;

//           case KeyEvent.VK_W: setWrap(); event.consume(); return;

           //case KeyEvent.VK_C: event.consume(); return;
           default: break;
         }

   	    if(Editable)
   	     {
   	      switch(keypressed)
   	      {
   		   case KeyEvent.VK_S: save();  event.consume(); return;
   	       case KeyEvent.VK_R: replace(); event.consume(); return;
   		   case KeyEvent.VK_E: textModel.clearEOL(); event.consume(); return;
	       //case KeyEvent.VK_I:  insert(); event.consume(); return;
   	       case KeyEvent.VK_Y: textModel.clearLine(); event.consume(); return;
   	       case KeyEvent.VK_Z: undo();  event.consume(); return;
   	       default: break;
       	  }
       	 //event.consume();
       	 return;
       	}
       } // is control down

    	if(keypressed == KeyEvent.VK_F3)
    	 {
    	   findSelection();
    	   event.consume();
    	   return;
    	 }


        if(event.isActionKey()) { dispTip(""); return; }

        // touches ordinaires
        if(Editable)
        {
         if(isEdited() == false)
         {
           dispTip("");
           setEdited(true);
          // debug("key pressed " + keypressed);
         }
         char c = event.getKeyChar();
         char newc = new Character(c).charValue();
       	 if (c != newc) event.consume();
       	 event.setKeyChar(newc);
        }
	}




  /**
   * Adds a listener to receive action events
   */
   ActionListener actionListener = null;

   public void addActionListener(ActionListener l)
    {
     actionListener = AWTEventMulticaster.add(actionListener, l);
    }

   KeyListener keyListener;

   public void addKeyListener(KeyListener k)
   {
     keyListener = AWTEventMulticaster.add(keyListener, k);
   }

   public void sendKey(KeyEvent event)
   {
    if( keyListener != null)
     keyListener.keyPressed(event);
   }

   public void actionPerformed(ActionEvent e)
    {
     sendSource(e.getActionCommand());
    }

   public void sendSource(String command)
   {
    if(actionListener != null)
     actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, command));
   }


   DMessageBar messageBar = null;
   History history = null;

   public void setMessage(DMessageBar mb) { messageBar = mb; }
   public void setHistory(History h) { history = h; }

   public void dispTip(String str)
   {
    if(messageBar != null) messageBar.setText(" " + str);
   }

   public void message(String mess)
   {
     dispTip(mess);
     if(history != null) history.tell(mess);
   }

} // class


/*
	void Paste_Action()
	{
		public void keyReleased(java.awt.event.KeyEvent event)
		{
			Object object = event.getSource();
			if (object == CDWindow.this)
				CDWindow_KeyRelease(event);
		}

		int stt = area.getCaretPosition();

        area. insert(m_Clipboard, stt);
		Message("Clipboard content inserted at caret");
	}

	void Copy_Action()
	{
		m_Clipboard = area.getSelectedText();
		Message("Selection copied into clipboard");
	}

	void Cut_Action()
	{
    	area.Keypress(new KeyEvent(
    	     this,
    	     Event.KEY_PRESS,
    	     0,
    	     KeyEvent.CTRL_MASK,
    	     KeyEvent.VK_X
    	     ));


		m_Clipboard = area.getSelectedText();
		int stt = area.getSelectionStart();
		int lst = area.getSelectionEnd();
        area. replaceRange("", stt, lst);

		Message("Selected text moved into clipboard");
	}
*/

