/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import common.*;

/**
   La position du curseur retourne par getCaretPosition
   de mme que les autres paramtres de localisation,
   est fausse.
   Elle est suprieure du nombre de lignes qui prcde.
   On peut la calculer en dcomptant les codes EOL
   dans la reprsentation du texte par getText()

   Comme si TextArea avait 2 codes de fin de ligne
   contre 1 pour TextComponent.

   La longueur de la chaine obtenue par getText()
   de mme est infrieure d'une valeur qui est le nombre de lignes
   contenues, dans la chane.

   Toutes les fonctions qui utilisent getText et de localisation
   doivent dcompter les lignes et ajouter le nombre obtenu pour
   fournir la localisation sur l'cran.

   Ainsi, si on utilise la taille du texte comme butoir, pour
   le parcourir, la longueur de la chaine est insuffisante.
*/

public class TextModel
{
  TextArea ta;
  String UndoText;
  int UndoPos;

  static String separator = System.getProperty("line.separator");

  public TextModel(TextArea ta)
  {
    this.ta = ta;
    clearUndo();

	//{{INIT_CONTROLS
		//}}
}


  /** Taille du texte */
  public int size()
  {
   return ta.getText().length();
  }

 /**
   Founit le nombre de code EOL entre le dbut du texte
   et la position donne prise  l'affichage.
   */

  public int EOLCount(int cursor)
   {
	 int depl = 0;
	 String text = ta.getText();

	 for(int i = 0; i <= cursor; i++)
        if(text.charAt(i) == '\n') depl++;

	 return depl;
   }


  /** Ajuste la largeur d'un texte */

  public void wrapIt(int cols)
  {
    int len = size();
    String text = ta.getText();

    int seplen = separator.length();
    int newsize = len + (len / cols)  + 5;
    StringBuffer newtext = new StringBuffer(newsize);


    char sep1 = '\0';
    char sep2 = '\0';

    try {
     sep1 = separator.charAt(0);
     sep2 = separator.charAt(1);
    }
    catch(Exception se) { sep2 = '\0'; }


    //System.out.println("size=" + len);
    //System.out.println("cols=" + cols);
    //System.out.println("newsize=" + newsize);
    //System.out.println("sep1=" + sep1 + " " + new Integer(sep1).toString());
    //System.out.println("sep2=" + sep2 + " " + new Integer(sep2).toString());


    int pos = 0;

	for(int i = 0; i < len; i++)
	{
	 try
	 {
       char c = text.charAt(i);
	   newtext.append(c);
	   pos++;
       if (c == sep1)
        if ((text.charAt(i+1) == sep2) || (sep2 == '\0'))
          pos = 0;

	   if (pos >= cols)
	     {
	      //System.out.println(pos + " = " + cols);
	      int j = i;
	      while ((j > 0) && (pos > 0))
	      {
	        if (newtext.charAt(j) == ' ')
	        {
/*
  	         newtext.setCharAt(j, sep2);
  	         if(sep2 != '\0') newtext.insert(j, sep2);
  */
             newtext.insert(j + 1, separator);
  	         break;
  	        }

  	        j--;
  	        pos--;
  	      } //while
	      pos = 0;
	     } //if
	 }
	 catch(StringIndexOutOfBoundsException e)
	  {
	    e.printStackTrace();
  	    System.out.println(i + " > " + newsize);
	  }
	}

    ta.setText(new String(newtext));

  }

  /**
     Ajuste la position du curseur
     en tenant compte des EOL  dduire
     */


  public int getRow()
   {
	 int cursor  = getCaretPosition();
	 String text = ta.getText();
	 int line = 0;

	 for(int i = 0; i < cursor; i++)
       if(text.charAt(i) == '\n') line++;

	 return line + 1;
   }


  /**
     Position du dbut de la ligne courante
    */
  public int getStartOfLine()
   {
	 int cursor  = getCaretPosition();
	 String text = ta.getText();
	 int stt = 0;

	 for(int i = 0; i < cursor; i++)
       if(text.charAt(i) == '\n') stt = i;

	 return stt + 1;
   }

   /**
      Calcule la ligne et la colonne courantes
      */
   public int[] getLineCol()
   {
	 int cursor  = getCaretPosition();
	 String text = ta.getText();
	 int stt = 0;
	 int line = 0;
	 int ret[] = new int[2];

	 for(int i = 0; i < cursor; i++)
	 {
       if(text.charAt(i) == '\n')
       {
        stt = i;
        line++;
       }
     }

	 ret[0] = line + 1;
	 ret[1] = (cursor - stt) + 1;
     return ret;
   }

  /**
     Nombre de lignes effectif
    */

  public int countLines()
   {
	 String text = ta.getText();
	 int lines = 0;
     int i = 0;

     boolean again = true;
     while(again)
     try {
      if(text.charAt(i++) == '\n') lines++;
     } catch(StringIndexOutOfBoundsException e) { again = false; }

	 return lines;
   }


  public int getPosition(int TApos)
  {
   String text = ta.getText();

   int i = 0;
   while(i < TApos)
    if(text.charAt(i++) == '\n') TApos --;

    return TApos;
  }

  /**
     Fonctions surcharges pour ajuster le curseur
    */

  public int getCaretPosition()
   {
//	return getPosition(ta.getCaretPosition());
	return ta.getCaretPosition();
   }

  public int getSelectionStart()
  {
//   return getPosition(ta.getSelectionStart());
    return ta.getSelectionStart();
  }

  public int getSelectionEnd()
  {
//   return getPosition(ta.getSelectionEnd());
   return ta.getSelectionEnd();
  }


  public boolean isSelected()
  {
   int start = ta.getSelectionStart();
   int end   = ta.getSelectionEnd();

   return(end > start);
  }

  public String getSelectedText()
  {
   int start = ta.getSelectionStart();
   int end   = ta.getSelectionEnd();
   String text = ta.getText();

   int i = 0;
   while(i < start)
     if(text.charAt(i++) == '\n')  { start--; end--; }

   while(i < end)
     if(text.charAt(i++) == '\n') end--;

   return(text.substring(start, end));
  }


  /**
     Equivalences
     Indpendantes de la position
     ou nouvelles fonctions
    */

  public void setText(String text) { ta.setText(text); }

  public String getText() { return ta.getText(); }

  public void insert(String text, int pos) { ta.insert(text, pos); }

  public void insert(String text) { ta.insert(text, getCaretPosition()); }


  /** Remplace par la chaine dsign, celles entre start et end */
  public void replace(String newstr, int start, int end)
  {
    String text = ta.getText();
    int tlen = text.length();

    StringBuffer n = new StringBuffer(tlen + newstr.length());
    n.append(text.substring(0, start));
    n.append(newstr);
    n.append(text.substring(end, tlen));

    ta.setText(new String(n));
  }


  public void replace(String newstr)
  {
    int start = getSelectionStart();
    int end   = getSelectionEnd();

    replace(newstr, start, end);
  }


  /** Remplace par la chaine dsign, la chaine slectionne  */
  public void replaceSelection(String newstr)
  {
    int start = getPosition(ta.getSelectionStart());
    int end   = getPosition(ta.getSelectionEnd());

    ta.replaceRange(newstr, start, end);
  }



  public int getLastPosition()
  {
    return ta.getText().length() + countLines();
  }

  /**
    Remove a block of text
    Start and end included
    */

  public void remove(int start, int end)
   {
     UndoText = getText().substring(start, end + 1);
     UndoPos = start;
	 ta.replaceRange("", start, end);
   }

  /**
     Efface fin de ligne
   */
  public void clearEOL()
   {
	 int cursor  = getCaretPosition();
	 String text = ta.getText();

     int j = cursor;
     while(j < text.length() && (text.charAt(j) != '\n')) j++;

     remove(cursor, j);
   }

  /**
     Efface ligne
   */
  public void clearLine()
   {
	 int cursor  = getCaretPosition();
	 String text = ta.getText();

     int j = cursor;
     int i = cursor;
     while(i > 0 && (text.charAt(i) != '\n')) i--;
     while(j < text.length() && (text.charAt(j) != '\n')) j++;

     remove(i, j);
   }


  /**
     Restaure ce qui est effac par ^E et ^Y
  */
  public void undo()
  {
    if(UndoText.length() > 0)
    {
     ta.insert(UndoText, UndoPos);
     clearUndo();
    }
  }

  public void clearUndo()
  {
    UndoText = "";
    UndoPos = 0;
  }



// ---------------------------- JUSTIFICATION -------

/*
String JUSTIFYBUF;
static char EOP = '\n';
static char EOL = '\n';
static char EOT = 0x1a;
bytes bBuffer;

int copySelection()
{
  bBuffer = getSelectedText().getBytes();
  return bBuffer.length;
}

void center(int textWidth)
{
 int i, j;
 int p, q;
 char c;
 char flag;
 byte newBuffer[] = new byte[2048];
 byte newLine[];
 String newString = "";
 int nStart;
 int tLen;

 int size = copySelection();

 flag = 0;
 i = 0;
 j = 0;

 while(i < size)
  {
   newLine = new byte[2048];
   lStart = i;

   while(bBuffer[i] != EOP)
   {
    c = bBuffer[i++];
    if(c == 9) c = ' ';
    if(flag || (c != ' '))
     {
      flag = 1;
      if((c != ' ') || (bBuffer[i] != ' ')) bBuffer[j++] = c;
     }
   }

   tLen = j - lStart;

   int padding = (textWidth + j + 1) / 2;

   for(p = 0; p < padding; p++) newLine[p] = ' ';
   for(q = 0; q < tLen; q++) newLine[p + q] = bBuffer[lStart + q];
   newString = newString + new String(newLine, 0, p + q);
 }

 replaceSelection(newString);
}

// Ligne justifie  gauche


void leftJustify()
{
 char line[2048];
 int i, j;
 char c;

 int lLen = copySelection();

 i = 0;
 j = 0;
 while(i < lLen)
  {
   c = bBuffer[i++];
   if(c == 9) c = ' ';
   if((c != ' ') || (bBuffer[i] != ' ')) line[j++] = c;
 }

  replaceSelection(new String(line, 0, j));
}




int blanksCount(int size)
{
 int i = 0;

 while(size > 0)
  if(bBuffer[size--] == ' ') i++;

 return(i);
}

int getEOP(int start, int end)
{
 byte c;
 int i = start;

 boolean found = false;

 while(!found && i < end)
 {
  c = bBuffer[i++];

  if(c == EOT) found = true;
  if(c == '.') found = true;
  if(c == '?') found = true;
  if(c == '!') found = true;
  if(c == ')') found = true;
  if(c == '}') found = true;
  if(c == ']') found = true;
  if(c == EOL && (bBuffer[i] == EOL)) found = true; // 2 fin de ligne
 }

 return(i);
}




int compress(int oldsize)
{
 int i;
 int newsize;
 char c;
 char previous;
 char decalage;

 i         = 0;   	//  source
 newsize   = 0;  	//  destination
 previous  = 0;
 decalage  = 0;

 while(i < oldsize)
  {
   c = (int) CURSOR[i++];
   if(!c)     c = ' ';
   if(c == 9) c = ' ';
   if((c == ' ') && (previous == ' '))
    {
     if(i <= CURSOR_X)  decalage++;
    }
   else
    {
     JUSTIFYBUF[newsize++] = c;
     previous = c;
    }
  }

 return(newsize);
}


int justify(char mode)
{
 char c, pass;
 register int i;
 int  newsize;
 int  oldsize;
 char huge *newbuffer;
 char huge *buffer;
 char huge *hugeadr;
 int  pris, lineremain;
 int  espaces;            // nombre de blancs au dpart entre les mots
 int  blk_to_add;         // nombre de blancs  ajouter
 int  decalage;
 int  souligne;

 if(!WRAP) goto bye;
 clearkey();
 oldsize = FOUND_EOP();

 if(mode == JUSTIFY_LINE)
  if(oldsize < TEXT_WIDTH)   goto bye;
 if(oldsize > MAX_PARAGRAPH) goto bye;

// compresse d'abord

 lineremain = compress(oldsize);    // taille du paragraphe

 buffer     = JUSTIFYBUF;
 newbuffer  = (char huge *) BOXBUF;
 newsize    = 0;
 pass       = 0;
 decalage   = 0;

// transfert d'une part de largeur d'cran

nextpart:
 souligne   = (buffer[0] == 4);	      // drapeau soulignement
 i          = TEXT_WIDTH + souligne;  // largeur corrige
 blk_to_add = 0;                      // nombre de blancs  ajouter

 if(lineremain <= i)   // reste du paragraphe infrieur  largeur de texte
  {
   hugeadr = hugepint(newbuffer, newsize);      // BOXBUF incrment
   memmove((char *) hugeadr, (char *) buffer, lineremain); // transfert brut
   newsize += lineremain;
   goto ok;
  }

// Ici, longueur de ligne avec blancs gale largeur du texte

// On trouve le dernier blanc et on dcompte ceux qu'il faudra insrer

 espaces  = just_blanks(i - 1);  		      // i - 1 avant
 while(i && buffer[i] != ' ') { i--; blk_to_add++; }  // trouver la fin
 if(!i) goto bye;            			      // ligne pleine

 pris = i;       	/// pris pointe sur le dernier blanc
 i    = 0;          // pointeur sur le texte de dpart
 while(i < pris)   	// de 0 jusqu'au dernier car. blanc exclu
  {
   c = buffer[i];
   newbuffer[newsize++] = c;
   if(i)
    if(c == ' ')
     {
      if(blk_to_add >= espaces)      // Second insr si (ins > espaces)
       {
	newbuffer[newsize++] = ' ';  // On insre
	espaces--;                   // Un de moins  transfrer
	blk_to_add--;                // Un de moins  insrer
	if(!pass)                    // Si sur ligne dite
	 if(i < (CURSOR_X + souligne)) decalage++;
       }

      if(blk_to_add)                 // Premier insr tant qu'il en reste
       {
	newbuffer[newsize++] = ' ';
	blk_to_add--;
	if(!pass)
	 if(i < (CURSOR_X + souligne)) decalage++;
       }
     }
   i++;
  }

 newbuffer[newsize++] = 0;
 buffer 	= hugepint(buffer, pris);
 lineremain    -= pris;
 if(*buffer == ' ')            // Enlever blanc en dbut de ligne
  { buffer++; lineremain--; }
 pass          	= 1;
 goto nextpart;

ok:
 CURSOR_X += decalage;         // Ajouter blancs insrs avant curseur
 DELETE_STRING(oldsize);
 INSERT_STRING(newbuffer, newsize);

 if(CURSOR_X > TEXT_WIDTH)     // passage ligne suivante
  {
   CURSOR    = hugepint(CURSOR, TEXT_WIDTH + 1 + souligne);
   CURSOR_X -= TEXT_WIDTH;     // Retour en dbut de ligne
   CURSOR_X--;                 // zro ajout en taille,  dduire
   CURSOR_Y++;                 // ligne suivante
  }

 CURSOR_X  += blk_to_add;     // Nbr car trans. = nbr blancs ajouts

 MARK_START = 0;
 MARK_END   = 0;

bye:
 return(1);
}



void justification()
{
//  MOVE_START();
//  justify(JUSTIFY_LINE);
//  MOVE_START();
//  DE_X_A_CURSOR();
}

void whole_justify()
{
 char savw = WRAP;
 WRAP = 1;
 CURSOR = BUFFER;


while(CURSOR < POSEND)
 {
  if(bBuffer[CURSOR] == EOL) { CURSOR++; }
  else
  {
   if(bBuffer[CURSOR] == EOT) break;
   justify(JUSTIFY_TEXT);
   CURSOR = CURSOR + FOUND_EOP() + 1;
  }
 }

  WRAP = savw;
  return;
}


void wholeFormat()
{
 int i;
 int len = bBuffer.length;


 while(CURSOR < POSEND)
 {
  if(bBuffer[CURSOR] != EOL)
  {
   if(bBuffer[CURSOR] == EOT) return;
   i = LOCATE_EOP_OLD();
   if(i > MAX_PARAGRAPH) i = len;
   CURSOR = CURSOR + i;
  }

  if(BBuffer[CURSOR-1] != EOP)
  {
   if(OPEN_SPACE(1))
   {
    bBuffer[CURSOR++] = EOP;
   }
  }
  CURSOR++;
 }

}

*/
	//{{DECLARE_CONTROLS
	//}}
}
